/**
 * Implement RS232 on windows plarform.
 * 
 * (C) Copyright (C) 2013, Datecs Ltd.
 * 
 * No part of this software may be used, stored, compiled, reproduced,
 * modified, transcribed, translated, transmitted, or transferred, in any form
 * or by any means whether electronic, mechanical, magnetic, optical,
 * or otherwise, without the express prior written permission. 
 */

#ifndef _CRT_SECURE_NO_DEPRECATE
#define _CRT_SECURE_NO_DEPRECATE 1
#endif

#include "rs232.h"

HANDLE rs232_open(const char *port, int baudRate, int byteSize, int parity, int stopBits)
{
	HANDLE handle = CreateFileA(port, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL);

	// Update DCB settings
	if (handle != INVALID_HANDLE_VALUE) {
		DCB dcb;

		GetCommState(handle, &dcb);
		dcb.BaudRate		= baudRate;
		dcb.ByteSize		= byteSize;
		dcb.Parity			= parity; //ODDPARITY
		dcb.StopBits		= stopBits;
		dcb.fDtrControl		= DTR_CONTROL_DISABLE;//DTR_CONTROL_ENABLE; //DTR_CONTROL_DISABLE
		dcb.fOutxCtsFlow	= FALSE;
		dcb.fDsrSensitivity = FALSE;	
		dcb.fRtsControl		= RTS_CONTROL_DISABLE;	
		dcb.fOutX			= FALSE;
		dcb.fInX			= FALSE;
		dcb.XonChar			= 0x11;
		dcb.XoffChar		= 0x13;
		dcb.fAbortOnError   = FALSE;
		dcb.fNull			= FALSE;
		dcb.fBinary			= TRUE;

		if (!SetCommState(handle, &dcb)) {
			CloseHandle(handle);
			handle = INVALID_HANDLE_VALUE;
		}
	}

	// Update timeouts
	if (handle != INVALID_HANDLE_VALUE) {	
		COMMTIMEOUTS cto;		
		
		cto.ReadIntervalTimeout			= MAXDWORD;
		cto.ReadTotalTimeoutConstant	= 0;
		cto.ReadTotalTimeoutMultiplier	= 0;
		cto.WriteTotalTimeoutConstant	= 0;
		cto.WriteTotalTimeoutMultiplier	= 1000;	

		if (!SetCommTimeouts(handle, &cto)) {
			CloseHandle(handle);
			handle = INVALID_HANDLE_VALUE;	
		}
	}

	return handle;
}

void rs232_close(HANDLE handle)
{
	CloseHandle(handle);
}

BOOL rs232_read(HANDLE handle, void *buf, unsigned int len,  unsigned int *nbr)
{	
	return ReadFile(handle, buf, (DWORD)len, nbr, NULL);
}

BOOL rs232_write(HANDLE handle, const void *buf, unsigned int len, unsigned int *nbw)
{
	return WriteFile(handle, buf, (DWORD)len, nbw, NULL);
}

BOOL rs232_purge(HANDLE handle)
{
	return PurgeComm(handle, PURGE_RXCLEAR | PURGE_TXCLEAR);
}

