/**
 * Pinpad API definition.
 * 
 * (C) Copyright (C) 2013, Datecs Ltd.
 * 
 * No part of this software may be used, stored, compiled, reproduced,
 * modified, transcribed, translated, transmitted, or transferred, in any form
 * or by any means whether electronic, mechanical, magnetic, optical,
 * or otherwise, without the express prior written permission. 
 */

#ifndef _PINPAD_H
#define _PINPAD_H

#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

// The type of error code
#define PPSDK_ERR  unsigned int

// PPSDK error numbers
#define PPSDK_ENOERR		0	/* No error */
#define PPSDK_EGENERAL		1	/* Some error */
#define PPSDK_EINVCMD		2	/* No valid command or subcommand code */
#define PPSDK_EINVPAR		3	/* Invalid paremeter */
#define PPSDK_EINVADR		4	/* Address is outside limits */
#define PPSDK_EINVVAL		5	/* Value is outside limits */
#define PPSDK_EINVLEN		6	/* Length is outside limits */
#define PPSDK_ENOPERMIT		7	/* The action is not permit in current state */
#define PPSDK_ENODATA		8	/* There is no data to be returned */
#define PPSDK_ETIMEOUT		9	/* Timeout is occur */
#define PPSDK_EKEYNUM		10	/* Invalid key number */
#define PPSDK_EKEYATTR		11	/* Invalid key attributes(usage) */
#define PPSDK_EINVDEVICE	12	/* Calling of non-existing device */
#define PPSDK_ENOSUPPORT	13	/* (No used in this FW version) */
#define PPSDK_EPINLIMIT		14	/* Pin entering limit exceed */
#define PPSDK_EFLASH		15	/* Some error in flash commands */
#define PPSDK_EHARD			16	/* Some hardware error */
#define PPSDK_EINVCRC		17	/* Invalid checksum */
#define PPSDK_ECANCEL		18	/* The button "CANCEL" is pressed */
#define PPSDK_EINVSIGN		19	/* Invalid signature */
#define PPSDK_EINVHEAD		20	/* Invalid data in header */
#define PPSDK_EINVPASS		21	/* Incorrent password */
#define PPSDK_EKEYFORMAT	22	/* Invalid key format */
#define PPSDK_ESCR			23	/* Some error in smart card reader */
#define PPSDK_EHAL			24	/* Error code is returned from HAL functions */
#define PPSDK_EINVKEY		25	/* Invalid key (may be no pressent) */
#define PPSDK_ENOPINDATA	26	/* The PIN length is <4 or >12 */
#define PPSDK_EINVREMAINDER	27	/* Issuer or ICC key invalid remainder length */
#define PPSDK_ENOINIT		28	/* (No used in this FW version) */
#define PPSDK_ELIMIT		29	/* (No used in this FW version) */
#define PPSDK_EINVSEQ		30	/* (No used in this FW version) */
#define PPSDK_ENOPERM		31	/* The action is not permited */
#define PPSDK_ENOTMK		32	/* TMK is not loaded. The action cannot be executed */
#define PPSDK_EINVKEK		33	/* Wrong kek format */
#define PPSDK_EDUBKEY		34	/* Dublicated key */
#define PPSDK_EKBD			35	/* General keyboard error */
#define PPSDK_EKBDNOCAL		36	/* The keyboard is no calibrated */
#define PPSDK_EKBDBUG		37	/* Keyboard bug detected */
#define PPSDK_EBUSY			38	/* The device is busy, tray again */
#define PPSDK_ETAMPERED		39	/* The devicete in security tampered */
#define PPSDK_EEMSR			40	/* Error in encrypted head */
#define PPSDK_EACCEPT		41	/* The button "OK" is pressed */

////////////////////////////////////////////////////////////////////////////////
// Generic function
////////////////////////////////////////////////////////////////////////////////

// Event codes.
#define PPSDK_EVENT_SWITCH1_ON     1
#define PPSDK_EVENT_SWITCH1_OFF    2
#define PPSDK_EVENT_SWITCH2_ON     3
#define PPSDK_EVENT_SWITCH2_OFF    4
#define PPSDK_EVENT_BARCODE        5
#define PPSDK_EVENT_MAGNETIC_CARD  6
#define PPSDK_EVENT_SMART_CARD_IN  7
#define PPSDK_EVENT_SMART_CARD_OUT 8
#define PPSDK_EVENT_BLUETOOTH_DATA 9	
#define PPSDK_EVENT_RF             10
#define PPSDK_EVENT_EMV2           11

#define PPSDK_KEY_TR31       0x00

// EMV2 message codes
#define PPSDK_EMV2_MESSAGE_NOT_WORKING               0x0000
#define PPSDK_EMV2_MESSAGE_APPROVED                  0x0003
#define PPSDK_EMV2_MESSAGE_DECLINED                  0x0007
#define PPSDK_EMV2_MESSAGE_PLEASE_ENTER_PIN          0x0009
#define PPSDK_EMV2_MESSAGE_ERROR_PROCESSING          0x000F
#define PPSDK_EMV2_MESSAGE_REMOVE_CARD               0x0010
#define PPSDK_EMV2_MESSAGE_PRESENT_CARD              0x0015
#define PPSDK_EMV2_MESSAGE_IDLE                      0x0014
#define PPSDK_EMV2_MESSAGE_PROCESSING                0x0016
#define PPSDK_EMV2_MESSAGE_CARD_READ_OK_REMOVE       0x0017
#define PPSDK_EMV2_MESSAGE_TRY_OTHER_INTERFACE       0x0018
#define PPSDK_EMV2_MESSAGE_CARD_COLLISION            0x0019
#define PPSDK_EMV2_MESSAGE_SIGN_APPROVED             0x001A
#define PPSDK_EMV2_MESSAGE_ONLINE_AUTHORISATION      0x001B
#define PPSDK_EMV2_MESSAGE_TRY_OTHER_CARD            0x001C
#define PPSDK_EMV2_MESSAGE_INSERT_CARD               0x001D
#define PPSDK_EMV2_MESSAGE_CLEAR_DISPLAY             0x001E
#define PPSDK_EMV2_MESSAGE_SEE_PHONE                 0x0020
#define PPSDK_EMV2_MESSAGE_PRESENT_CARD_AGAIN        0x0021
#define PPSDK_EMV2_MESSAGE_NA                        0x00FF

// EMV2 status codes 
#define PPSDK_EMV2_STATUS_NOT_READY         0x00
#define PPSDK_EMV2_STATUS_IDLE              0x01
#define PPSDK_EMV2_STATUS_READY_TO_READ     0x02
#define PPSDK_EMV2_STATUS_PROCESSING        0x03
#define PPSDK_EMV2_STATUS_CARD_READ_SUCCESS 0x04
#define PPSDK_EMV2_STATUS_ERROR_PROCESSING  0x05

// EMV2 event number
#define PPSDK_EMV2_EVENT_ONLINE_AUTHORIZATION  0x81
#define PPSDK_EMV2_EVENT_UPDATE_UI             0x82
#define PPSDK_EMV2_EVENT_COMPLETE              0x83
#define PPSDK_EMV2_EVENT_INTERFACE             0x85

// Pinpad configuration data structure
typedef struct {
	
	/**
	 * A callback function, use to read data from Pinpad. (Required)	 
     *
	 * @param param a pointer to a variable to be passed to the function.
	 * @param out a buffer that receives data.
	 * @param outMaxSize the maximum length of received data.
	 * @param nbr a pointer to value that receives number of bytes stored in output buffer.
	 * @param timeout the time to complete read operation.
	 *
	 * @return On success returns 0; otherwise, returns a positive error number.
	 */
	PPSDK_ERR (*read)(void* param, unsigned char *out, unsigned int outMaxLen, unsigned int *nbr, unsigned int timeout);

	/**
	 * A callback function, use to write data to Pinpad. (Required)	 
	 * 
	 * @param param a pointer to a variable to be passed to the function.
	 * @param in a buffer that contains data to be write.
	 * @param inLen data length.
	 * @param nbr a pointer to value that receives the actual number of bytes written.
	 *
	 * @return On success returns 0; otherwise, returns a positive error number.
	 */
	PPSDK_ERR (*write)(void* param, const unsigned char in[], unsigned int inLen, unsigned int *nbr);

	/**
	 * A callback function, use to notify when event is received. (Optional)
	 *
	 * @param handler a pointer to the instance handler. 
	 * @param param a pointer to a variable to be passed to the function.	 
	 * @param evn the event number. The value can be one of PPSDK_EVENT_* constant type values.
	 * @param data pointer to the buffer that contains event data.
	 * @param dataLen data buffer length.
	 */
	void (*notify)(void *handler, void* param, unsigned int evn, const unsigned char data[], unsigned int dataLen);	

	/**
	 * A callback function, that is called when online authorization is required.
	 *	
	 * @param handler a pointer to the instance handler. 
	 * @param param a pointer to a variable to be passed to the function.	 
	 * @param data the output data.
     * <pre>
     *   - 'ONLINE' event - Tags specified in template 0xE5, Tag 0xC1
     * </pre>
	 * @param dataLen data buffer length.
	 */
	void (*emv2_online_authorize)(void *handler, void* param, const unsigned char data[], unsigned int dataLen);	

	/**
     * Update user interface.
     * 
	 * @param handler a pointer to the instance handler. 
	 * @param param a pointer to a variable to be passed to the function.	 
     * @param id the message identifier, one of PPSDK_EMV2_MESSAGE_* constant values.
     * @param status the status. It can be one of PPSDK_EMV2_STATUS_* constant values or
     *            negative if not present.
     * @param hold the time that the message has to stay on the display without being
     *            interrupted. It can be a positive unit of 100ms ( Example: Value = 5, means
     *            Hold time = 500ms ), or negative value if not present.
     */
	void (*emv2_update_ui)(void *handler, void* param, int message, int status, int hold);

	/**
     * On transaction finish.
     * 
	 * @param handler a pointer to the instance handler. 
	 * @param param a pointer to a variable to be passed to the function.	 
     * @param data the output data:
     * <pre>
     *  - Transaction Result (Tag 0xC1) - 2 bytes (big endian);
     *    - CL_RESULT_APPROVED              0x0800
     *    - CL_RESULT_DECLINED              0x0400
     *    - CL_RESULT_TRY_ANOTHER_INTERFACE 0x0100 
     *    - CL_RESULT_END_APPLICATION       0x0000        
     *
     * - CVM (Tag 0xC2) - 1 byte - Present only if transaction is Approved;
     *    - CL_CVM_ONLINE_PIN                0x01
     *    - CL_CVM_CONF_CODE_VERIFIED        0x02
     *    - CL_CVM_OBTAIN_SIGNATURE          0x03
     *    - CL_CVM_NO_CVM                    0x04
     *
     * - Tags specified in template 0xE5, Tag 0xC2
     * </pre>
	 * @param dataLen data buffer length.
     */
	void (*emv2_complete)(void *handler, void* param, const unsigned char data[], unsigned int dataLen);	


} ppsdk_config;

/**
 * Initialises Pinpad.
 *
 * @param config a pointer to the configuration data.
 * @param param a pointer to a variable to be passed to functions.
 * @param handler a pointer to variable that receives instance handler.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_init(const ppsdk_config *config, void *param, void **handler);

/**
 * Release Pinpad.
 *
 * Must be called at the end of any successful 'ppsdk_init' function call.
 *
 * @param handler a pointer to the instance handler. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_release(void *handler);

/**
 * Transmit command to Pinpad.
 *
 * @param handler a pointer to the instance handler. 
 * @param command command identifier.
 * @param in a buffer that contains input data.
 * @param inLen input data length.
 * @param out a buffer that receives output data.
 * @param outMaxLen output data max length.
 * @param nbr a pointer to variable that receives number of bytes stored in output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_transmit(void *handler, unsigned char command, 
						 const unsigned char in[], unsigned int inLen, 
                         unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

/**
 * Pull device to receive events. 
 *
 * This method can be called in thread to provide event handling. 
 *
 * @param handler a pointer to the instance handler. 
 * @param wait the time to wait for event data.
 *
 * @return the error text.
 */
PPSDK_ERR ppsdk_pull(void *handler, unsigned int wait);

/**
 * Returns text representation of error number.
 *
 * @param err the error number.
 *
 * @return the error text.
 */
const char *ppsdk_getErrorText(unsigned int err);

////////////////////////////////////////////////////////////////////////////////
// GET IDENTIFICATION INFO AND SERIAL NUMBER
////////////////////////////////////////////////////////////////////////////////

#define PPSDK_DEVICE_TYPE_LOADER      0x10 // Loader
#define PPSDK_DEVICE_TYPE_FIRMWARE    0x11 // Firmware
#define PPSDK_DEVICE_TYPE_APPLICATION 0x12 // Example application

// Pinpad identification structure.
typedef struct {
	unsigned char devtype;   // Type of device
	unsigned int version;  // Application version
	char name[16];  // Application name
	char iser[16];  // INNOVA serial number
	unsigned int dser;     // DATECS serial number
	unsigned int klver;    // Key loading rsa key version
} ppsdk_info;

/**
 * Returns information about Pinpad.
 *
 * @param handler a pointer to the instance handler. 
 * @param info a pointer to structure that receives information data. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_getInfo(void *handler, ppsdk_info *info);

////////////////////////////////////////////////////////////////////////////////
// 3DES KEYS MANAGMENT, CRYPTO, HASH
////////////////////////////////////////////////////////////////////////////////

/**
 * Encrypt data with 3DES ECB algorithm.
 *
 * @param handler a pointer to the instance handler. 
 * @param key a key number (1 ~ 49).
 * @param in a buffer that contains data to be encrypt.
 * @param inLen input data length.
 * @param out a buffer that receives encrypted data.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoEncryptECB(void *handler, unsigned char key, const unsigned char in[], unsigned int inLen, unsigned char out[]);

/**
 * Decrypt data with 3DES ECB algorithm.
 *
 * @param handler a pointer to the instance handler. 
 * @param key a key number (1 ~ 49).
 * @param in a buffer that contains data to be decrypt.
 * @param inLen input data length.
 * @param out a pointer to buffer that receives decrypted data.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoDecryptECB(void *handler, unsigned char key, const unsigned char *in, unsigned int inLen, unsigned char out[]);

/**
 * Encrypt data with 3DES CBC algorithm.
 *
 * @param handler a pointer to the instance handler. 
 * @param key a key number (1 ~ 49).
 * @param in a buffer that contains data to be encrypt.
 * @param inLen input data length.
 * @param out a pointer to buffer that receives encrypted data.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoEncryptCBC(void *handler, unsigned char key, const unsigned char in[], unsigned int inLen, unsigned char out[]);

/**
 * Decrypt data with 3DES CBC algorithm.
 *
 * @param handler a pointer to the instance handler. 
 * @param key a key number (1 ~ 49).
 * @param in a buffer that contains data to be decrypt.
 * @param inLen input data length.
 * @param out a buffer that receives decrypted data.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoDecryptCBC(void *handler, unsigned char key, const unsigned char in[], unsigned int inLen, unsigned char out[]);

// Hash type definitions
#define PPSDK_HASH_SHA1		(unsigned char)1 // SHA1
#define PPSDK_HASH_SHA256	(unsigned char)2 // SHA256
#define PPSDK_HASH_MD5		(unsigned char)3 // MD5 (may be not pressent in some versions)
#define PPSDK_HASH_HDES		(unsigned char)4 // HDES (may be not pressent in some versions)
#define PPSDK_HASH_RIPEMD	(unsigned char)5 // RIPEMD (may be not pressent in some versions)

/**
 * Calculate hash data.
 *
 * @param handler a pointer to the instance handler. 
 * @param type the hash algorithm. The value must be one of PPSDK_HASH_* constant type values.
 * @param in a buffer that contains data to be hash.
 * @param inLen input data length.
 * @param out a buffer that receives hashed data. The length of returned data may vary depends of hash algorithm.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoHash(void *handler, unsigned char type, const unsigned char in[], unsigned int inLen, unsigned char out[]);

/**
 * Save 3DES keys to flash.
 * 
 * @param handler a pointer to the instance handler.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoSaveKeysToFlash(void *handler);

// Crypto modes
#define PPSDK_MODE_ECBENC 0 // Encrypt with 3DES ECB
#define PPSDK_MODE_ECBDEC 1 // Decrypt with 3DES ECB
#define PPSDK_MODE_CBCENC 2 // Encrypt with 3DES CBC
#define PPSDK_MODE_CBCDEC 3	// Decrypt with 3DES CBC

/**
 * Encrypt/decrypt data with 3DES.
 * 
 * @param handler a pointer to the instance handler.
 * @param mode a crypto mode. The value must be one of PPSDK_MODE_* constant type values. 
 * @param key a key number (1 ~ 49).
 * @param iv a buffer that contains initial vector (IV).
 * @param variant a buffer that contains key variant data.
 * @param in a buffer that contains data to be encrypt/decrypt.
 * @param inLen input data length.
 * @param out a buffer that receives encrypted/decrypted data.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoCrypt(void *handler, unsigned char mode, unsigned char key, const unsigned char iv[8], 
					  const unsigned char variant[16], const unsigned char in[], unsigned int inLen, unsigned char out[]);

/**
 * Delete key.
 * 
 * @param handler a pointer to the instance handler.
 * @param key a key number (1 ~ 49). 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoDeleteKey(void *handler, unsigned char key);

/**
 * Get key check value (KCV).
 *  
 * @param handler a pointer to the instance handler. 
 * @param key a key number (1 ~ 49). 
 * @param kcv a buffer that receives key check value.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoGetKCV(void *handler, unsigned char key, unsigned char kcv[8]);

/**
 * Set DUKPT initial key.
 *  
 * @param handler a pointer to the instance handler. 
 * @param kek a key encryption key number (1 ~ 49).
 * @param key a key number (1 ~ 49). 
 * @param data a buffer that contains key data.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoSetDUKPT(void *handler, unsigned char kek, unsigned char key, const unsigned char data[16]);

// Key usage definitions
#define PPSDK_KEY_KEKDATAENC    0x02 // Key encryption key for data encrypt
#define PPSDK_KEY_KEKDATADEC    0x03 // Key encryption key for data decrypt
#define PPSDK_KEY_KEKPIN        0x04 // Key encryption key for PIN
#define PPSDK_KEY_KEKMAC1       0x05 // Key encryption key for MAC1
#define PPSDK_KEY_KEKMAC3       0x06 // Key encryption key for MAC3
#define PPSDK_KEY_KEKMAC1A      0x07 // Key encryption key for MAC1A
#define PPSDK_KEY_KEKDUKPT      0x08 // Key encryption key for DUKPT
#define PPSDK_KEY_DATAENC       0x12 // Key for data encrypt
#define PPSDK_KEY_DATADEC       0x13 // Key for data decrypt
#define PPSDK_KEY_PIN           0x14 // Key for encrypt PIN
#define PPSDK_KEY_MAC1          0x15 // Key for calculate MAC1
#define PPSDK_KEY_MAC3          0x16 // Key for calculate MAC3
#define PPSDK_KEY_MAC1A         0x17 // Key for calculate MAC1A
#define PPSDK_KEY_DUKPT         0x18 // Key for DUKPT

// MAC calculation modes
#define PPSDK_MAC_MAC1  0x01 // ISO 9797-1 MAC Algorithm 1
#define PPSDK_MAC_MAC3  0x03 //ISO 9797-1 MAC Algorithm 3
#define PPSDK_MAC_MAC1A 0x0B // ISO 16609 MAC algorithm 1

/**
 * Calculate MAC.
 * 
 * @param handler a pointer to the instance handler.
 * @param mode a MAC calculation mode. The value must be one of PPSDK_MAC_* constant type values. 
 * @param key a key number (1 ~ 49).
 * @param iv a buffer that contains initial vector (IV).
 * @param variant a buffer that contains key variant data.
 * @param in a buffer that contains data to be calculated.
 * @param inLen the input data length.
 * @param mac a buffer that receives calculated MAC.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoCalculateMAC(void *handler, unsigned char mode, unsigned char key, const unsigned char iv[8],
							 const unsigned char variant[16], const unsigned char in[], unsigned int inLen, unsigned char mac[8]);

/**
 * Exchange (set) 3DES ECB encrypted key.
 * 
 * If usage = 0 and version = 0 then TR31 method is used.
 *
 * @param handler a pointer to the instance handler. 
 * @param kek a key encryption key number (1 ~ 49).
 * @param key a key number to store new key (1 ~ 49).
 * @param usage key usage. The value must be one of PPSDK_KEY_* constant type values. 
 * @param version key version.
 * @param data a buffer that contains 3DES key data.
 * @param kcv a buffer that receives key check value(KCV).
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoExchangeECBKey(void *handler, unsigned char kek, unsigned char key, unsigned char usage, 
							unsigned char version, const unsigned char data[16], unsigned char kcv[8]);

/**
 * Exchange (set) 3DES CBC encrypted key.
 * 
 * If usage = 0 and version = 0 then TR31 method is used.
 *
 * @param handler a pointer to the instance handler. 
 * @param kek a key encryption key number (1 ~ 49).
 * @param key a key number to store new key (1 ~ 49).
 * @param usage key usage. The value must be one of PPSDK_KEY_* constant type values. 
 * @param version key version.
 * @param data a buffer that contains 3DES key data.
 * @param kcv a buffer that receives key check value(KCV).
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_cryptoExchangeCBCKey(void *handler, unsigned char kek, unsigned char key, unsigned char usage, 
							unsigned char version, const unsigned char data[16], unsigned char kcv[8]);


////////////////////////////////////////////////////////////////////////////////
// CA KEYS MANAGMENT, EMV COMMANDS
////////////////////////////////////////////////////////////////////////////////

/**
 * Import certification authority (CA) keys.
 *  
 * @param handler a pointer to the instance handler.  
 * @param key a key number to store new key (0 ~ 29).
 * @param rid a registered application provider identifier (RID).
 * @param idx key index. 
 * @param mod a buffer that contains key module.
 * @param modLen key module length.
 * @param exp a buffer that contains key exponent.
 * @param expLen key exponent length.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_caImportKeys(void *handler, unsigned char key, const unsigned char rid[5], unsigned char idx, 
							 const unsigned char mod[], unsigned int modLen, 
							 const unsigned char exp[], unsigned int expLen);

/**
 * Writes certification authority (CA) keys to flash.
 *  
 * @param handler a pointer to the instance handler.   
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_caWriteKeysToFlash(void *handler);

/**
 * Get info about certification authority (CA) keys.
 *  
 * @param handler a pointer to the instance handler.
 * @param data a buffer that receives key data. The data contains records of 5 bytes
 *             RID and 2 bytes module length in little endian format. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_caGetCAKeyInfo(void *handler, unsigned char data[240]);

/**
 * Import issuer key. 
 *  
 * @param handler a pointer to the instance handler.
 * @param key a key number (0 ~ 29).
 * @param mod a buffer that contains key module.
 * @param modLen key module length.
 * @param exp a buffer that contains key exponent.
 * @param expLen key exponent length.
 * @param cert a buffer that contains certificate data.
 * @param certLen certificate data length.
 * @param out a buffer that receives decrypted certificate (certLen bytes).
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_caImportIssuerKey(void *handler, unsigned char key, 
								  const unsigned char mod[], unsigned int modLen, 
								  const unsigned char exp[], unsigned int expLen, 
								  const unsigned char cert[], unsigned int certLen,
								  unsigned char out[]);

/**
 * Import ICC key. 
 *  
 * @param handler a pointer to the instance handler.
 * @param type key type. The value must be one of: 
 *             0 - ICC
 *             1 - PIN 
 * @param mod a buffer that contains key module.
 * @param modLen key module length.
 * @param exp a buffer that contains key exponent.
 * @param expLen key exponent length.
 * @param cert a buffer that contains certificate data.
 * @param certLen certificate data length.
 * @param out a buffer that receives decrypted certificate (certLen bytes).
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_caImportICCKey(void *handler, unsigned char type, 
								  const unsigned char mod[], unsigned int modLen, 
								  const unsigned char exp[], unsigned int expLen, 
								  const unsigned char cert[], unsigned int certLen,
								  unsigned char out[]);

/**
 * Verify RSA key.
 *  
 * @param handler a pointer to the instance handler.
 * @param type key type. The value must be one of: 
 *             0 - Issuer key
 *             1 - ICC key
 * @param in a buffer that contains RSA key data.
 * @param inLen data length.
 * @param out a buffer that receives key data.
 * @param nbr a pointer to variable that receives number of bytes stored in output buffer. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_caRSAVerify(void *handler, unsigned char type, 
							const unsigned char in[], unsigned int inLen, 
							unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

/**
 * Generate DUKPT key (Online).
 *  
 * @param handler a pointer to the instance handler.
 * @param key a key number (0 ~ 1).
 * @param ksn a buffer that receives the DUKPT key serial number (KSN).
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_dukptGenerateKey(void *handler, unsigned char key, unsigned char ksn[16]);

/**
 * Calculate MAC (Online).
 *  
 * @param handler a pointer to the instance handler. 
 * @param iv a buffer that contains initialization vector (IV).
 * @param data a buffer that contains calculation data.
 * @param dataLen calculation data length.
 * @param mac a buffer that receives the calculated MAC.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_dukptCalculateMAC(void *handler, const unsigned char iv[8], 
								  const unsigned char data[], unsigned int dataLen, unsigned char mac[8]);

/**
 * Get key serial number (KSN) from last DUKPT generation.
 *  
 * @param handler a pointer to the instance handler.  
 * @param ksn a buffer that receives the DUKPT key serial number.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_dukptGetCurrentKSN(void *handler, unsigned char ksn[16]);

/**
 * Encrypt data with ECB DUKPT.
 *  
 * @param handler a pointer to the instance handler. 
 * @param iv a buffer that contains initialization vector (IV).
 * @param data a buffer that contains data to be encrypt.
 * @param dataLen data length.
 * @param out a buffer that receives the encrypted data.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_dukptEncryptECB(void *handler, const unsigned char iv[8], 
								const unsigned char data[], unsigned int dataLen, unsigned char out[]);

/**
 * Encrypt data with CBC DUKPT.
 *  
 * @param handler a pointer to the instance handler. 
 * @param iv a buffer that contains initialization vector (IV).
 * @param data a buffer that contains data to be encrypt.
 * @param dataLen data length.
 * @param out a buffer that receives the encrypted data.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_dukptEncryptCBC(void *handler, const unsigned char iv[8], 
								const unsigned char data[], unsigned int dataLen,  unsigned char out[]);


////////////////////////////////////////////////////////////////////////////////
// SMART CARD COMMANDS
////////////////////////////////////////////////////////////////////////////////
// Smart card reader slot definitions 
#define PPSDK_SC_SLOT_MAIN 0 // Main slot.    
#define PPSDK_SC_SLOT_SAM  1 // SAM slot.

// Smart card voltage definitions   
#define PPSDK_SC_VOLTAGE_3  3 // 3V smart card
#define PPSDK_SC_VOLTAGE_5  5 // 5V smart card

// Smart card protocol definitions
#define PPSDK_SC_PROTOCOL_T0 0 // T0 protocol
#define PPSDK_SC_PROTOCOL_T1 1 // T1 protocol

/**
 * Initializes smart card  module
 *  
 * @param handler a pointer to the instance handler. 
 * @param slot a slot to operate with. The value must be one of PPSDK_SC_SLOT_* constant type values.
 * @param voltage smart card operating voltage. The value must be one of PPSDK_SC_VOLTAGE_* constant type values.
 * @param protocol smart card communication protocol. The value must be one of PPSDK_SC_PROTOCOL_* constant type values.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_scInit(void *handler, unsigned char slot, unsigned char voltage, unsigned char protocol);

/**
 * Powers on the smart card, resets it and returns answert-to-reset (ATR).
 *  
 * @param handler a pointer to the instance handler. 
 * @param slot a slot to operate with. The value must be one of PPSDK_SC_SLOT_* constant type values.
 * @param atr a buffer that receives answert-to-reset data (ATR).
 * @param atrMaxLen the maximum length of received ATR data.
 * @param nbr a pointer to variable that receives number of bytes stored in ATR buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_scCardPowerOn(void *handler, unsigned char slot, unsigned char atr[], unsigned int atrMaxLen, unsigned int *nbr);

/**
 * Powers off smart card.
 *  
 * @param handler a pointer to the instance handler. 
 * @param slot a slot to operate with. The value must be one of PPSDK_SC_SLOT_* constant type values.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_scCardPowerOff(void *handler, unsigned char slot);

/**
 * Checks if there is a card in the reader.
 *  
 * @param handler a pointer to the instance handler. 
 * @param slot a slot to operate with. The value must be one of PPSDK_SC_SLOT_* constant type values.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_scCheckCard(void *handler, unsigned char slot);

/**
 * Performs APDU command in the card.
 *  
 * @param handler a pointer to the instance handler. 
 * @param slot a slot to operate with. The value must be one of PPSDK_SC_SLOT_* constant type values.
 * @param in a buffer that contains input data.
 * @param inLen input data length.
 * @param out a buffer that receives output data.
 * @param outMaxLen the maximum length of received data.
 * @param nbr a pointer to variable that receives number of bytes stored in output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_scCAPDU(void *handler, unsigned char slot, const unsigned char in[], unsigned char inLen, 
						unsigned char out[], unsigned char outMaxLen, unsigned int *nbr);

/**
 * Shutdown smart card module.
 *  
 * @param handler a pointer to the instance handler. 
 * @param slot a slot to operate with. The value must be one of PPSDK_SC_SLOT_* constant type values. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_scShutdown(void *handler, unsigned char slot);

////////////////////////////////////////////////////////////////////////////////
// MAGNETIC CARD COMMANDS
////////////////////////////////////////////////////////////////////////////////

/**
 * Enables reading of magnetic cards.
 *  
 * When started, magnetic card events will be fired. Reading of magnetic cards is enabled by
 * default upon connect, so need to call this function unless you have disabled it.
 *
 * @param handler a pointer to the instance handler. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_msStart(void *handler);

/**
 * Checks if magnetic card data is available.
 *   
 * @param handler a pointer to the instance handler. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_msPull(void *handler);

/**
 * Get data of scanned card.
 *
 * This method is obsolate. 
 *   
 * @param handler a pointer to the instance handler. 
 * @param out a buffer that receives output data.
 * @param outMaxLen the maximum length of received data.
 * @param nbr a pointer to variable that receives number of bytes stored in output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_msGetCardData(void *handler, unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

/**
 * Disable reading of magnetic cards.
 *   
 * @param handler a pointer to the instance handler. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_msStop(void *handler);

/**
 * Get card data encrypted with RSA key. 
 * 
 * The return data is encrypted in format: unicid[4] trancnt[4] msrdata[?] 
 *
 * @param handler a pointer to the instance handler. 
 * @param out a buffer that receives output data.
 * @param outMaxLen the maximum length of received data.
 * @param nbr a pointer to variable that receives number of bytes stored in output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_msGetEncryptedCardDataRSA(void *handler, unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

/**
 * Get card data encrypted with RSA key. 
 * 
 * The encrypted data is in format: sha256[0x20] unicid[4] modlen[2] expllen[2] mod[modlen] exp[explen]
 *
 * @param handler a pointer to the instance handler. 
 * @param key a key number (1 ~ 49).
 * @param data a buffer that contains encrypted RSA block.
 * @param dataLen input buffer length. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_msLoadRSAkey(void *handler, unsigned char key, const unsigned char data[], unsigned int dataLen);

/**
 * Get manual entered card data. 
 * 
 * The return data is encrypted in format: unicid[4] trancnt[4] msrdata[?] 
 *
 * @param handler a pointer to the instance handler. 
 * @param out a buffer that receives output data.
 * @param outMaxLen the maximum length of received data.
 * @param nbr a pointer to variable that receives number of bytes stored in output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_msGetManualCardData(void *handler, unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

/**
 * Get card data encrypted with 3DES CBC. 
 * 
 * The return data is encrypted in format: ran[4] id[4] len[2] data[?] crc[2] pad[?], where:
 *    ran:	random number
 *    id:	unique session number
 *    len:	data length (big endian)
 *    data:	track data
 *    crc:	crcccitt (init value 0xFFFF)
 *    pad:	0..7 zero bytes padding
 *
 * @param handler a pointer to the instance handler. 
 * @param key a key number (1 ~ 49). If key number is FF, then the last generated DUKPT key is used.
 * @param id a unique session identifier.
 * @param out a buffer that receives output data.
 * @param outMaxLen the maximum length of received data.
 * @param nbr a pointer to variable that receives number of bytes stored in output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_msGetEncryptedCardDataCBC(void *handler, unsigned char key, unsigned int id,
										  unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

/**
 * Get masked card data.
 *
 *  This function must be executed after calling ppsdk_msGetEncryptedCardDataCBC function.
 *
 * @param handler a pointer to the instance handler. 
 * @param unmaskedDigitsAtStart the number of digits to show in clear text at the start of the
 *            PAN, range from 0 to 6.
 * @param unmaskedDigitsAtEnd the number of digits to show in clear text at the end of the PAN,
 *            range from 0, to 4.
 * @param unmaskedDigitsAfter the the number of digits to show in clear text after the PAN,
 *            range from 0, to 7.
 * @param out a buffer that receives output data.
 * @param outMaxLen the maximum length of received data.
 * @param nbr a pointer to variable that receives number of bytes stored in output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_msGetMaskedCardData(void *handler, unsigned int unmaskedDigitsAtStart, 
									unsigned int unmaskedDigitsAtEnd, unsigned int unmaskedDigitsAfter,
									unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

////////////////////////////////////////////////////////////////////////////////
// DISPLAY / KEYBOARD COMMANDS
////////////////////////////////////////////////////////////////////////////////

/**
 * Draw text string.
 * 
 * @param handler a pointer to the instance handler. 
 * @param text a buffer that contains NULL terminated text string.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiDrawString(void *handler, const char text[]);

/**
 * Read key (hardware button).
 * 
 * @param handler a pointer to the instance handler. 
 * @param key a pointer to value that receives button number.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiReadKey(void *handler, unsigned char *key);

// Language code definitions
#define PPSDK_LANG_ENGLISH     0
#define PPSDK_LANG_BULGARIAN   1
#define PPSDK_LANG_ESPANIOL    2
#define PPSDK_LANG_RUSSIAN     3
#define PPSDK_LANG_ROMANIAN    4
#define PPSDK_LANG_FRENCH      5
#define PPSDK_LANG_FINISH      6
#define PPSDK_LANG_SWEDISH     7
#define PPSDK_LANG_CATALUN     8

/**
 * Enables numeric mode. 
 * 
 * This function disable status line
 *
 * @param handler a pointer to the instance handler. 
 * @param x the X coordinate.
 * @param y the Y coordinate.
 * @param lang the language code. The value must be one of PPSDK_LANG_* constant type values.
 * @param text a buffer that contains NULL terminated text string.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiEnableNumericMode(void *handler, unsigned int x, unsigned int y, unsigned int lang, const char text[]);

/**
 * Disable numeric mode. 
 *  
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiDisableNumericMode(void *handler);

// Font type definitions.
#define PPSDK_FONT_6X8  0 // Max 21x8 chars
#define PPSDK_FONT_8X16 1 // Max 16x4 chars
#define PPSDK_FONT_4X6  2 // Max 32x10 chars

// Code table definitions.			
#define PPSDK_CODETABLE_LATIN1    0 // Latin1 (West European)
#define PPSDK_CODETABLE_LATIN2    1 // Latin2 (East European)
#define PPSDK_CODETABLE_LATIN3    2 // Latin3 (South European)
#define PPSDK_CODETABLE_LATIN4    3 // Latin4 (North European)
#define PPSDK_CODETABLE_CYRILLIC  4 // Cyrillic
#define PPSDK_CODETABLE_ARABIC    5 // Arabic
#define PPSDK_CODETABLE_GREEK     6 // Greek
#define PPSDK_CODETABLE_HEBREW    7 // Hebrew
#define PPSDK_CODETABLE_LATIN5    8 // Latin5 (Turkish)
#define PPSDK_CODETABLE_LATIN6    9 // Latin6 (Nordic) 

/**
 * Enables numeric mode. 
 * 
 * This function disable status line
 *
 * @param handler a pointer to the instance handler. 
 * @param x the X coordinate.
 * @param y the Y coordinate.
 * @param ch the horizontal size in characters.
 * @param cv the vertical size in characters.
 * @param font the font type. The value must be one of PPSDK_FONT_* constant type values.
 * @param codetable the code table. The value must be one of PPSDK_CODETABLE_* constant type values.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiOpenTextWindow(void *handler, unsigned int x, unsigned int y, unsigned int ch, unsigned int cv, 
								 unsigned char font, unsigned char codetable);

/**
 * Get keyboard control.
 *  
 * @param handler a pointer to the instance handler.  
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiGetKeyboardControl(void *handler);

/**
 * Release keyboard control.
 *  
 * @param handler a pointer to the instance handler.  
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiReleaseKeyboardControl(void *handler);

/**
 * Set key tone.
 *  
 * @param handler a pointer to the instance handler.  
 * @param freq the tone frequency in Hz.
 * @param duration the tone duration in miliseconds.
 * @param volume the tone volume form 0 to 100%.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiSetKeyTone(void *handler, unsigned int freq, unsigned int duration, unsigned char volume);

// Pin encryption mode
#define PPSDK_PIN_MODE_PLAIN     0 // Plaintext PIN
#define PPSDK_PIN_MODE_ENCRYPTED 1 // Encrypted PIN

/**
 * Enter PIN.
 *  
 * @param handler a pointer to the instance handler.  
 * @param mode the pin ecryption mode. The value must be one of PPSDK_PIN_MODE_* constant type values.
 * @param sw a pointer to variable that rereceives card status.
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiEnterPIN(void *handler, unsigned int mode, unsigned short *sw);

/**
 * Enter PIN.
 *  
 * @param handler a pointer to the instance handler.  
 * @param x the echo X coordinate.
 * @param y the echo Y coordinate.
 * @param echo the echo symbol: '*', '+' or '-'.
 * @param timeout the timeout in seconds. Valid values are between 10s and 180s.
 * @param sw a buffer that receives card status.
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiEnterPIN2(void *handler, unsigned int x, unsigned int y, char echo, unsigned int timeout, const char msg[]);

// PIN block format definitions
#define PPSDK_PIN_FORMAT_ISO0  0x04 // ISO 0
#define PPSDK_PIN_FORMAT_ISO1  0x05 // ISO 1
#define PPSDK_PIN_FORMAT_ISO3  0x0D // ISO 3

/**
 * Verify PIN offline.
 *  
 * @param handler a pointer to the instance handler.  
 * @param mode the pin ecryption mode. The value must be one of PPSDK_PIN_MODE_* constant type values.
 * @param pinfmt the pin block format. The value must be one of PPSDK_PIN_FORMAT_* constant type values.
 * @param key a key number to store new key (1 ~ 49).
 * @param variant a buffer that contains key variant data.
 * @param pan a buffer that contains NULL terminated PAN number.
 * @param pin a abuffer that receives the encrypted pin. 
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiVerifyPINOnline(void *handler, unsigned char mode, unsigned char pinfmt, unsigned char key, 
								  unsigned char variant[16], const char pan[], unsigned char pin[8]);

// Color definitions
#define PPSDK_COLOR_WHITE	0 // White color
#define PPSDK_COLOR_BLACK	1 // Back color
#define PPSDK_COLOR_INVERTED 2 // Inverted color

/**
 * Fill rectangle.
 *  
 * @param handler a pointer to the instance handler.  
 * @param x the X coordinate.
 * @param y the Y coordinate.
 * @param width the rectangle width.
 * @param height the rectange height.
 * @param color the color. The value must be one of PPSDK_COLOR_* constant type values.
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiFillRectangle(void *handler, unsigned int x, unsigned int y, 
								unsigned int width, unsigned int height, unsigned int color);

/**
 * Set display contrast.
 *  
 * @param handler a pointer to the instance handler.  
 * @param contrast the display contrast value.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiSetContrast(void *handler, unsigned int contrast);

/**
 * Draw pixel.
 *  
 * @param handler a pointer to the instance handler.  
 * @param x the X coordinate.
 * @param y the Y coordinate. 
 * @param color the color. The value must be one of PPSDK_COLOR_* constant type values.
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiDrawPixel(void *handler, unsigned int x, unsigned int y, unsigned int color);

/**
 * Draw bitmap.
 *  
 * @param handler a pointer to the instance handler.  
 * @param x the X coordinate.
 * @param y the Y coordinate.
 * @param width the bitmap width.
 * @param height the bitmap height.
 * @param data a buffer that contains bitmap data (x * y / 8 bytes).
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiDrawBitmap(void *handler, unsigned int x, unsigned int y, 
								unsigned int width, unsigned int height, const unsigned char data[]);


// Animation object definitions
#define PPSDK_ANIM_INSERT_CARD  0 // Enter card
#define PPSDK_ANIM_REMOVE_CARD  1 // Remove card
#define PPSDK_ANIM_WAIT         2 // Wait
#define PPSDK_ANIM_DROP         3 // Drop
				
// Play mode definitions
#define PPSDK_PLAY_SINGLE   0 // Single play
#define PPSDK_PLAY_REPEAT   1 // Repeat playing

/**
 * Start animation.
 *  
 * @param handler a pointer to the instance handler.  
 * @param x the X coordinate.
 * @param y the Y coordinate.
 * @param anim the animation object. The value must be one of PPSDK_ANIM_* constant type values.
 * @param mode the animation play mode. The value must be one of PPSDK_PLAY_* constant type values. 
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiStartAnimation(void *handler, unsigned int x, unsigned int y, unsigned char anim, unsigned char mode);

/**
 * Stop animation.
 *  
 * @param handler a pointer to the instance handler.  
 * @param anim the animation object. The value must be one of PPSDK_ANIM_* constant type values.
 * @param mode the animation play mode. The value must be one of PPSDK_PLAY_* constant type values.
 * @param x the X coordinate.
 * @param y the Y coordinate. 
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiStopAnimation(void *handler, unsigned char anim);

/**
 * Set function button text.
 *  
 * @param handler a pointer to the instance handler.  
 * @param button the button number from 1 to 3. 
 * @param text a buffer that contains NULL terminated text.
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiSetButtonText(void *handler, unsigned char button, const char text[]);

/**
 * Show initial screen.
 * 
 * @param handler a pointer to the instance handler.  
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiInitScreen(void *handler);

/**
 * Enter PIN in non blocking mode.
 *  
 * @param handler a pointer to the instance handler.  
 * @param x the echo X coordinate.
 * @param y the echo Y coordinate.
 * @param echo the echo symbol: '*', '+' or '-'.
 * @param timeout the timeout in seconds. Valid values are between 10s and 180s.
 * @param msg  a buffer that contains NULL terminated text message.
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiEnterPINAsync(void *handler, unsigned int x, unsigned int y, char echo, unsigned int timeout, const char msg[]);

/**
 * Check if PIN is entered.
 *  
 * This function must be execute after calling ppsdk_uiEnterPINAsync.
 *
 * @param handler a pointer to the instance handler.  
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiCheckPIN(void *handler);

/**
 * Cancel non blocking PIN entry.
 *   
 * @param handler a pointer to the instance handler.  
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_uiCancelPIN(void *handler);

////////////////////////////////////////////////////////////////////////////////
// SYSTEM COMMANDS
////////////////////////////////////////////////////////////////////////////////
// A date and time structure, using individual members.
typedef struct {
	unsigned char year;    // The year. The valid values are 13(2013), 14(2014)...
	unsigned char month;   // The month. The valid values are 0(Jan), 1(Feb)...
	unsigned char day;     // The day of month. The valid values are from 1 to 31.
	unsigned char hour;    // The hour. The valid values are from 1 to 31.
	unsigned char minute;  // The minute. The valid values are from 0 to 59.
	unsigned char second;  // The second. The valid values are from 0 to 59.
} ppsdk_date;

/**
 * Get current device date and time. 
 *   
 * @param handler a pointer to the instance handler. 
 * @param clock a pointer to structure that receives current device date and time. 
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_sysGetDate(void *handler, ppsdk_date *date);

/**
 * Set device date and time. 
 *   
 * @param handler a pointer to the instance handler. 
 * @param clock a pointer to structure that contains new device date and time. 
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_sysSetDate(void *handler, const ppsdk_date *date);

/**
 * Pley beep sound.
 *   
 * The sounds are played in background.
 *
 * Up to 16 commands can be executed consecutively.
 *
 * @param handler a pointer to the instance handler. 
 * @param freq the tone frequency in Hz.
 * @param duration the tone duration in miliseconds.
 * @param volume the tone volume form 0 to 100%.
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_sysBeep(void *handler, unsigned int freq, unsigned int duration, unsigned char volume);

// LED type definitions
#define PPSDK_LED_RED   0  // The RED color led.
#define PPSDK_LED_GREEN 1  // The GREEN color led.

/**
 * Set led on and off.
 *   
 * @param handler a pointer to the instance handler. 
 * @param led the led type. The value must be one of PPSDK_LED_* constant type values. 
 * @param pattern the led pattern, where 0 is led off and 0xFFFF is led on;
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_sysSetLED(void *handler, unsigned char led, unsigned int pattern);

/**
 * Show status line on and off.
 *   
 * @param handler a pointer to the instance handler. 
 * @param value the visibility state, where 0 is hide and 1 is visible. 
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_sysShowStatus(void *handler, unsigned char value);

/**
 * Play melody.
 *   
 * The data format is "XY XY .. XY 00", where:
 *     X: duration (1 ~ 9)
 *     Y: note (1 ~ 0x0C)
 *     if X = 0x00 then Y = tempo (1 ~ 0x0F)
 *     if X = 0x0F then Y = volume (1 ~ 0x0A)
 *     if X = 0x0E then Y = oktave (1 ~ 0x0B)
 *		
 *     Note
 *     ----
 *     0     1    2    3    4    5    6    7    8    9   10   11   12
 *     pause do   do#  re   re#  mi   fa   fa#  so   so# la   la#  si
 *     
 *     Duration
 *     --------
 *     1    2    3    4    5    6    7    8    9
 *     1  3/4  1/2  3/8  1/4 3/16  1/8 3/32 1/16
 *     
 *     Oktave
 *     ------
 *     1    2    3    4    5    6    7    8    9   10   11
 *     ssc  scon con  grea smal 1    2    3    4    5    6
 *     
 *     Volume
 *     ------
 *     1    2    3    4    5    6    7    8    9   10
 *     10%  20%  30%  40%  50%  60%  70%  80%  90% 100%
 *     		
 *     Example
 *     -------
 *     FAE65253305556305253705556705B5A705253705A3930787673713300
 *     
 * @param handler a pointer to the instance handler. 
 * @param data a buffer that contains tone data.
 * @param dataLen data length.
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_sysPlayMelody(void *handler, const unsigned char data[], unsigned int dataLen);

/**
 * Erase external flash page.
 *   
 * The pages size is 128K.
 *
 * The address for remote FW update is 0x200000.
 *
 * @param handler a pointer to the instance handler. 
 * @param address the memory address in page form 0 to 0x3DFFFE.
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_sysEraseFlash(void *handler, unsigned int address);

/**
 * Writes data to external flash.
 *    
 * @param handler a pointer to the instance handler. 
 * @param address the memory address in page form 0 to 0x3DFFFE.
 * @param data a buffer that contains data to be written.
 * @param dataLen data length.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_sysWriteFlash(void *handler, unsigned int address, const unsigned char data[], unsigned int dataLen);

// Status bits definitions
#define PPSDK_STATUS_CHARGING        1  // The bit that indicates battery charging.
#define PPSDK_STATUS_LOWBATTERY      2  // The bit that indicates low battery.
#define PPSDK_STATUS_VERYLOWBATTERY  4  // The bit that indicates very low battery.
#define PPSDK_STATUS_EXTERNALPOWER   8  // The bit that indicates external power supply.

// Various hardware information
typedef struct {
	unsigned char capacity;  // The battery capacity in percents from 0 to 100%.
	unsigned char status;    // The battery status bit values. For additional information
	                         //  refer to PPSDK_STATUS_* bit constant values.
	unsigned char voltage;   // The battery voltage in 1/10th of Volt.
} ppsdk_hardinfo;

/**
 * Get hardware information.
 *    
 * @param handler a pointer to the instance handler. 
 * @param hardInfo a pointer to the structure that receives the hardware information. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_sysGetHardwareInfo(void *handler, ppsdk_hardinfo *hardInfo);

/**
 * Turn off device.
 *    
 * @param handler a pointer to the instance handler. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_sysTurnOff(void *handler);

// Enable event bit definitions
#define PPSDK_ENABLE_BUTTON1_PRESS     0x0002
#define PPSDK_ENABLE_BUTTON1_RELEASE   0x0004
#define PPSDK_ENABLE_BUTTON1           (ENABLE_BUTTON1_PRESS + ENABLE_BUTTON1_RELEASE)
#define PPSDK_ENABLE_BUTTON2_PRESS     0x0008
#define PPSDK_ENABLE_BUTTON2_RELEASE   0x0010
#define PPSDK_ENABLE_BUTTON2           (ENABLE_BUTTON2_PRESS + ENABLE_BUTTON2_RELEASE)
#define PPSDK_ENABLE_BARCODE           0x0020
#define PPSDK_ENABLE_MAGNETIC_CARD     0x0040
#define PPSDK_ENABLE_SMART_CARD_INSERT 0x0080
#define PPSDK_ENABLE_SMART_CARD_REMOVE 0x0100
#define PPSDK_ENABLE_SMART_CARD        (ENABLE_SMART_CARD_INSERT + ENABLE_SMART_CARD_REMOVE)

/**
 * Enable/disable automatic event notifications.
 *    
 * @param handler a pointer to the instance handler. 
 * @param mask the bit mask of events you want to enable. Teh value can be a combination of 
 *             PPSDK_ENABLE_* constant type values.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_sysEnableEvents(void *handler, unsigned int mask);

////////////////////////////////////////////////////////////////////////////////
// EMV2 COMMANDS (NECOM+ KERNEL)
////////////////////////////////////////////////////////////////////////////////

// EMV status codes definitions.
#define PPSDK_EMV_STATUS_SUCCESS                   0 // Operation successful.	
#define PPSDK_EMV_STATUS_LIST_AVAILABLE            1 // More than one matching applications found.	
#define PPSDK_EMV_STATUS_APPLICATION_AVAILABLE     2 // Only one matching application found.	
#define PPSDK_EMV_STATUS_NO_COMMON_APPLICATION     3 // No matching applications found.
#define PPSDK_EMV_STATUS_EASY_ENTRY_APP            4 // Easy Entry application.
#define PPSDK_EMV_STATUS_AMOUNT_NEEDED             5 // Amount is requested by the dynamic data authentication.
#define PPSDK_EMV_STATUS_RESULT_NEEDED             6 // Result needed.
#define PPSDK_EMV_STATUS_AUTH_COMPLETED            7 // Authentication is completed.
#define PPSDK_EMV_STATUS_AUTH_NOT_DONE             8 // Authentication was not performed.
#define PPSDK_EMV_STATUS_OFFLINE_PIN_PLAIN         9 // OFFLINE plain text pin is required.
#define PPSDK_EMV_STATUS_ONLINE_PIN               10 // ONLINE pin is required.
#define PPSDK_EMV_STATUS_OFFLINE_PIN_CIPHERED     11 // OFFLINE ciphered pin is required.
#define PPSDK_EMV_STATUS_BLOCKED_APPLICATION      12 // Explicit selection was done and blocked AIDs were found.
#define PPSDK_EMV_STATUS_TRANSACTION_ONLINE       13 // An online request should be done.
#define PPSDK_EMV_STATUS_TRANSACTION_APPROVED     14 // Transaction can be accepted offline.
#define PPSDK_EMV_STATUS_TRANSACTION_DENIED       15 // Transaction must be declined.
#define PPSDK_EMV_STATUS_CDA_FAILED               16 // CDA failed and the cryptogram got is not an AAC 
                                                     // or the data handed for DDA was not found.
#define PPSDK_EMV_STATUS_INVALID_PIN              17 // Incorrect PIN.
#define PPSDK_EMV_STATUS_INVALID_PIN_LAST_ATTEMPT 18 // Incorrect PIN, last attempt available only.
#define PPSDK_EMV_STATUS_FAILURE                  50 // Command failed, possibly due wrong input 
                                                     // parameters - wrong ATR, bit values, etc
#define PPSDK_EMV_STATUS_NO_DATA_FOUND            51 // Incoming data pointer is null or empty.
#define PPSDK_EMV_STATUS_SYSTEM_ERROR             52 // Internal system error occurred.
#define PPSDK_EMV_STATUS_DATA_FORMAT_ERROR        53 // Incorrect format found in the input parameters.
#define PPSDK_EMV_STATUS_INVALID_ATR              54 // Invalid ATR sequence, not according to specification.
#define PPSDK_EMV_STATUS_ABORT_TRANSACTION        55 // Severe error occurred transaction must be aborted.
#define PPSDK_EMV_STATUS_APPLICATION_NOT_FOUND    56 // AID not found in the card.
#define PPSDK_EMV_STATUS_INVALID_APPLICATION      57 // Application is not correct.
#define PPSDK_EMV_STATUS_ERROR_IN_APPLICATION     58 // Some error during read process.
#define PPSDK_EMV_STATUS_CARD_BLOCKED             59 // Status word got from the PSE selection indicates that 
	                                                 // the card is blocked.	
#define PPSDK_EMV_STATUS_NO_SCRIPT_LOADED         60 // No script loaded.
#define PPSDK_EMV_STATUS_TAG_NOT_FOUND            61 // Tag not found.		
#define PPSDK_EMV_STATUS_INVALID_TAG              62 // Tag cannot be read.	
#define PPSDK_EMV_STATUS_INVALID_LENGTH           63 // Length of the buffer is incorrect.	
#define PPSDK_EMV_STATUS_INVALID_HASH             64 // Error in the HASH verification.
#define PPSDK_EMV_STATUS_INVALID_KEY              65 //  No key was found to do the verification.
#define PPSDK_EMV_STATUS_NO_MORE_KEYS             66 // No more available locations for keys.
#define PPSDK_EMV_STATUS_ERROR_AC_PROCESS         67  // Error processing the AC generation.
#define PPSDK_EMV_STATUS_ERROR_AC_DENIED          68 // Status word got from the card is 6985.
#define PPSDK_EMV_STATUS_NO_CURRENT_METHOD        69
#define PPSDK_EMV_STATUS_RESULT_ALREADY_LOADED    70
#define PPSDK_EMV_STATUS_LAST_EMVKERNEL_ERR_CODE  70
#define PPSDK_EMV_STATUS_INVALID_REMAINDER        80
#define PPSDK_EMV_STATUS_INVALID_HEADER           81 // Invalid header.
#define PPSDK_EMV_STATUS_INVALID_FOOTER           82 //  Invalid footer.
#define PPSDK_EMV_STATUS_INVALID_FORMAT           83 // Invalid format.
#define PPSDK_EMV_STATUS_INVALID_CERTIFICATE      84 // Invalid certificate.
#define PPSDK_EMV_STATUS_INVALID_SIGNATURE        85 // Invalid signature.

/**
 * Through this command the kernel initializes itself and checks that everything is ready to
 * work properly.
 *
 * This command should be called only once at the Pinpad power up, additionally it should be the
 * first command call to be made with the exception of the get version data ppsdk_emvGetVersion.
 *
 * The main ICC reader should be open by the application, in this way the kernel is independent
 * of the additional usage that the application could do of the reader.
 *     
 * @param handler a pointer to the instance handler. 
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvInit(void *handler, unsigned short *sw);

/**
 * The command allows the application to release all the resources assigned in the
 * initialization process.
 * 
 * This command should be called only once at the end of the kernel usage, although normally it
 * wont be called if the application is waiting for an ICC insertion endlessly.
 * 
 * The main ICC reader wont be closed, in this way the kernel is independent of the additional
 * usage that the application could do of the reader.
 *     
 * @param handler a pointer to the instance handler. 
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvDeinitialize(void *handler, unsigned short *sw);

// EMV kernel version information
typedef struct {
	unsigned char version[5]; // Version number in ASCIIZ (ex. "1.00\x0").
	unsigned char release[3]; // Release in ASCIIZ (ex. "12\x0").
	unsigned char date[7];    // Version date in ASCIIZ (ex. "010203\x0")
} ppsdk_emv_ver;

/**
 * The command allows the application to gather details about the EMV kernel software version
 * and release date.
 * 
 * The data provided by this command can be used by the application to validate the expected
 * version of the kernel; this command can be called prior to kernel initialization.
 *     
 * @param handler a pointer to the instance handler. 
 * @param emvver a pointer to the strucure that receives kernel version information.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvGetVersion(void *handler, ppsdk_emv_ver *emvver, unsigned short *sw);

// Smart card reset type
#define PPSDK_SC_RESET_COLD 0 // Cold reset
#define PPSDK_SC_RESET_WARM 1 // Warm reset
/**
 * The command is in charge of validating the ATR sequence got from the card to ensure that is
 * fully EMV complaint and that obeys the rules stated in the specification.
 *  
 * @param handler a pointer to the instance handler. 
 * @param type the reset type. The value must be one of PPSDK_SC_RESET_* constant type values.
 * @param atr a buffer that contains ATR data.
 * @param atrLen ATR data length.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvValidateATR(void *handler, unsigned char type, 
							   const unsigned char atr[] , unsigned int atrLen, unsigned short *sw);

// Applicaiton selection method
#define PPSDK_EMV_SELECTION_METHOD_PSE 0  // PSE method
#define PPSDK_EMV_SELECTION_METHOD_AID 1  // AID method

// Applicaiton match criteria definitions.
#define PPSDK_EMV_MATCH_FULL	         1  // Full match
#define PPSDK_EMV_MATCH_PARTIAL_VISA     2  // Partial match - visa
#define PPSDK_EMV_MATCH_PARTIAL_EUROPAY  3  // Partial match EVROPAY

// Emv application.
typedef struct {
	unsigned char aid[16];   // The application AID.
	unsigned int aidLen;  // The AID length.
	unsigned char match;  // Applicaiton match criteria. The value must be one of PPSDK_EMV_MATCH_* 
	                      // constant type values. 
	char label[32];       // The NULL terminated label.
} ppsdk_emv_app;

/**
 * The command initiates the application selection process, loading the application list
 * supported by the terminal. The maximum number of application that can be loaded into the
 * kernel is up to 75. This number is only constrained by the max packet size that can be
 * exchanged on the port (2Kb).
 * 
 * Initially the command will inspect the incoming data to make sure that if data are provided
 * and that all the data related to terminal applications is valid. If no data has been provided
 * (the list is empty) the status PPSDK_EMV_STATUS_NO_DATA_FOUND will be returned, in the event of a 
 * format failure of the applications data the result got will be PPSDK_EMV_STATUS_DATA_FORMAT_ERROR. 
 * If during the internal procedure of the commands a system error occurs the command will return
 * with the status PPSDK_EMV_STATUS_SYSTEM_ERROR, on the other hand if the error occurs dealing with
 * the card or with the data got and the transaction must be aborted according to EMV specification, 
 * the result will be PPSDK_EMV_STATUS_ABORT_TRANSACTION. If the process can be completed correctly and 
 * the list is properly parsed and managed the status PPSDK_EMV_STATUS_SUCCESS will be returned.
 *  
 * @param handler a pointer to the instance handler. 
 * @param method the selection method. The value must be one of PPSDK_EMV_SELECTION_METHOD_* constant
 *               type values.
 * @param incblk a value to specify whether to inlucde block cards, where 1 is for include them.
 * @param app a list of structures that contains application information. 
 * @param count the number of application strucutres stored into the list.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvLoadAppList(void *handler, unsigned char method, const char incblk, 
							   const ppsdk_emv_app appList[], unsigned int count, unsigned short *sw);


/**
 * The command gets back the list of common applications supported by the terminal and the card,
 * actually this commands will end or resume the selection procedure.
 * 
 * Initially the command will check the provided data, if its empty string, the status
 * PPSDK_EMV_STATUS_NO_DATA_FOUND will be returned, if during the procedure any internal error occurs 
 * the status will be PPSDK_EMV_STATUS_SYSTEM_ERROR}. On the other hand, if the process and be completed
 * correctly the possible status returned will be: PPSDK_EMV_STATUS_LIST_AVAILABLE, 
 * PPSDK_EMV_STATUS_APPLICATION_AVAILABLE, PPSDK_EMV_STATUS_NO_COMMON_APPLICATION according to the number
 * of common applications found.
 * 
 * The application may know beforehand the number of common applications by retrieving the value
 * of the data item TAG_DF27_COMMON_APP_NUMBER.
 *
 * In the event of an application error that doesnt force to abort the transaction, this
 * command will be called again as many times as necessary while the list wont be empty.
 * Internally the Kernel will remove the wrong application so that the selection could be
 * resumed.
 *
 * @param handler a pointer to the instance handler. 
 * @param confirm a pointer to variable that receives if the confirmation is needed.
 * @param app a list of structures that receives application information. 
 * @param maxApp the maximum number of applications that can be received into list.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvGetCommonAppList(void *handler, unsigned char *confirm, 
									ppsdk_emv_app appList[], unsigned int maxApp, unsigned short *sw);

/**
 * Once an application has been selected, the next phase is to start the transaction with it by
 * issuing the GET PROCESSING command and analyzing the information got.
 * 
 * First the input data are checked, if empty the status PPSDK_EMV_STATUS_NO_DATA_FOUND} is returned, 
 * if the length of the AID is incorrect (greater than AID max length or less than TAG minimal length)
 * the status got will be PPSDK_EMV_STATUS_DATA_FORMAT_ERROR. If any internal error occurs during the
 * processing the status returned will be PPSDK_EMV_STATUS_SYSTEM_ERROR. Depending on the application 
 * type or status the codes PPSDK_EMV_STATUS_EASY_ENTRY_APP, PPSDK_EMV_STATUS_INVALID_APPLICATION} or
 * PPSDK_EMV_STATUS_BLOCKED_APPLICATION could be returned. If the transaction must be aborted due to a
 * processing error with the card or with the data got from it the status returned will be 
 * PPSDK_EMV_STATUS_ABORT_TRANSACTION. PPSDK_EMV_STATUS_APPLICATION_NOT_FOUND will be the status got 
 * if the AID provided cannot be found in the card. If everything is correct and the application can 
 * be initiated properly the status will be PPSDK_EMV_STATUS_SUCCESS.
 *
 * At this point of the transaction it could be possible to resume the application selection by
 * calling the ppsdk_emvGetCommonAppList command again, this will depend on the status got, normally
 * for PPSDK_EMV_STATUS_EASY_ENTRY_APP, PPSDK_EMV_STATUS_INVALID_APPLICATION or 
 * PPSDK_EMV_STATUS_BLOCKED_APPLICATION the selection should be resumed.
 *
 * @param handler a pointer to the instance handler. 
 * @param aid a buffer that contains selected application AID.
 * @param aidLen AID data length.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvInitialAppProcessing(void *handler, const unsigned char aid[] , unsigned int aidLen, unsigned short *sw);

/**
 * The command reads and validates the data informed in the AFL and that will be used along the
 * transaction.
 * 
 * If during the AFL data reading and validating an error occurs that commits the transaction to
 * be terminated, the status PPSDK_EMV_STATUS_ABORT_TRANSACTION will be returned. If the error allows the
 * application selection to be resumed, the status returned will be PPSDK_EMV_STATUS_ERROR_IN_APPLICATION.
 * If during the procedure any internal error occurs, PPSDK_EMV_STATUS_SYSTEM_ERROR will be returned. On 
 * the other hand, if everything is correct and the data can be extracted and validated, the status
 * PPSDK_EMV_STATUS_SUCCESS will be the value returned.
 *  
 * At this point of the transaction it could be possible to resume the application selection by
 * calling the ppsdk_emvGetCommonAppList command again, this will depend on the status got,
 * normally for PPSDK_EMV_STATUS_ERROR_IN_APPLICATION the selection should be resumed.
 * 
 * @param handler a pointer to the instance handler. 
 * @param tagList a buffer that contains list of tags.
 * @param count the number of tags in list. 
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvReadAppData(void *handler, const unsigned short tagList[], unsigned int count, unsigned short *sw);

/**
 * Through this command the card data is authenticated depending on the capabilities of the card
 * and the kernel. The method could be static or dynamic, in this case is completed here, or
 * combined that will be carried out later at the application cryptogram generation stage.
 * 
 * If the authentication can be performed (successfully or not) the command will return
 * PPSDK_EMV_STATUS_SUCCESS. If an internal error occurs the status got will be PPSDK_EMV_STATUS_SYSTEM_ERROR. 
 * PPSDK_EMV_STATUS_ABORT_TRANSACTION} will be returned if the transaction must be immediately terminated 
 * due to a severe error in the processing. If the check amount flag was enabled and the amount is 
 * one of the data items requested by the dynamic data authentication the status PPSDK_EMV_STATUS_AMOUNT_NEEDED
 * will be returned. If the authentication cannot be completed due to a missing CA public key, the 
 * status returned will be PPSDK_EMV_STATUS_INVALID_KEY.
 * 
 * PPSDK_EMV_STATUS_INVALID_KEY status code will let the application to detect and invalid configuration 
 * concerning the CA RSA public keys.
 * 
 * If the selected authentication method is the CDA, the verification of the CA public key
 * presence and the recovery of the issuer public key is done here prior to the actual CDA
 * verification to be done at the AC generation.
 * 
 * The reason for setting the checkAmount parameter to true is to allow the application to know
 * if the amount is required as part of the dynamic data used for the authentication. This can
 * be useful if the application plans to be sure that the actual amount will be used in the
 * process rather than a default value set to zero.
 *  
 * @param handler a pointer to the instance handler. 
 * @param check determine whether the amount is checked for the dynamic authentication. 
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvAutentication(void *handler, unsigned char check, unsigned short *sw);

/**
 * The command performs the restrictions processing related to application version, application
 * usage control and effective and expire dates.
 * 
 * If the process can be completed correctly the returned status will be PPSDK_EMV_STATUS_SUCCESS, 
 * if any internal error occurs the status will be PPSDK_EMV_STATUS_SYSTEM_ERROR instead.
 * 
 * To complete this process the kernel needs from the application the following data items to
 * have been provided prior to this command:
 *		TAG_9F09_APP_VERSION_NUMBER
 *		TAG_9F35_TERMINAL_TYPE
 *		TAG_9F40_ADD_TERM_CAPABILITIES
 *		TAG_9F1A_TERMINAL_COUNTRY_CODE
 *		TAG_9A_TRANSACTION_DATE
 *		TAG_9C_TRANSACTION_TYPE
 *  
 * @param handler a pointer to the instance handler.  
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvProcessRestrictions(void *handler, unsigned short *sw);

/**
 * The application risk control is done by this command, including floor limit checking, random
 * selection (only if offline is enabled) and velocity checking.
 * 
 * If the process can be completed correctly the returned status will be PPSDK_EMV_STATUS_SUCCESS, if 
 * any internal error occurs the status will be PPSDK_EMV_STATUS_SYSTEM_ERROR instead.
 *  
 * To complete this process the kernel needs from the application the following data items to
 * have been provided previously:
 *		TAG_DF02_RISK_AMOUNT (if offline enabled)
 *		TAG_81_AMOUNT_AUTHORISED_BINARY (if online only)
 *		TAG_9F19_FLOOR_LIMIT_CURRENCY (optional)
 *		TAG_9F1B_TERMINAL_FLOOR_LIMIT
 *		TAG_DF07_THRESHOLD_VALUE (if offline)
 *		TAG_DF08_TARGET_PERCENTAGE (if offline)
 *		TAG_DF09_MAX_TARGET_PERCENTAGE (if offline)
 *		TAG_5F2A_TRANSACTION_CURR_CODE (optional)
 *  
 * @param handler a pointer to the instance handler.  
 * @param force determine whether the process should be carried out despite of the AIP
 *              configuration.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvTerminalRisk(void *handler, unsigned char force, unsigned short *sw);

/**
 * The command starts or resumes the card holder authentication procedure, the current
 * verification method is communicated to the application.
 * 
 * The lists of methods and conditions is parsed and processed to identify what are the valid
 * ones according to the kernel capabilities the possible methods available are:
 * PPSDK_EMV_STATUS_OFFLINE_PIN_PLAIN, PPSDK_EMV_STATUS_ONLINE_PIN, PPSDK_EMV_STATUS_OFFLINE_PIN_CIPHERED.
 * If during the process an internal error occurs the status PPSDK_EMV_STATUS_SYSTEM_ERROR is returned,
 * if the transaction has to be terminated the status PPSDK_EMV_STATUS_ABORT_TRANSACTION will be
 * returned. If there are not more valid methods to be applied the status 
 * PPSDK_EMV_STATUS_AUTH_COMPLETED is set.
 *  
 * If a combination of methods is required by the card, pin verification plus signature, the
 * kernel directly checks if the latter is possible according to the capabilities, if so the
 * former is informed otherwise the next entry in the list will be processed.
 *
 * @param handler a pointer to the instance handler.  
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvGetAuthMethod(void *handler, unsigned short *sw);

// EMV authentication result definitions
#define PPSDK_EMV_AUTH_RESULT_SUCCESS          1 // The method result was successful.
#define PPSDK_EMV_AUTH_RESULT_FAILURE          2 // The method failed.
#define PPSDK_EMV_AUTH_FAIL_SEC_DEVICE_FAIL    3 // The secure device failed.
#define PPSDK_EMV_AUTH_FAIL_PIN_ENTRY_NOT_DONE 4 // The PIN entry was bypassed.
#define PPSDK_EMV_AUTH_FAIL_USER_CANCELLATION  5 // The PIN entry was canceled.

/**
 * Using this command the kernel gets the result of the previously informed verification method.
 * 
 * Firstly the value of the result informed must be checked, if its value is not a valid one the
 * status PPSDK_EMV_STATUS_DATA_FORMAT_ERROR will be returned. If the authentication process was not 
 * started and no method is currently active the status PPSDK_EMV_STATUS_NO_CURRENT_METHOD will be got 
 * by the application, if the result for the current method was already provided the status will be
 * PPSDK_EMV_STATUS_RESULT_ALREADY_LOADED. PPSDK_EMV_STATUS_SYSTEM_ERROR will be get by the application
 * if any internal error occurs during the processing. When everything is OK and the result can be stored
 * correctly the status sent back is PPSDK_EMV_STATUS_SUCCESS.
 * 
 * The actual verification method result according to EMV specification can be recovered by the
 * application at later stage by accessing the data item TAG_9F34_CH_VERIF_METHOD_RESULT.
 * 
 * @param handler a pointer to the instance handler.  
 * @param authRes authentication result code. The value must be one of PPSDK_EMV_AUTH_* constant type
 *                values.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvSetAuthResult(void *handler, unsigned char authRes, unsigned short *sw);

/**
 * The command allows the application to apply the offline PIN verification (plain text or
 * encrypted) method.
 *
 * Depending on the current PIN entry type (plain text or encrypted) is verified against the
 * card, if the PIN is no valid and is rejected the status PPSDK_EMV_STATUS_INVALID_PIN will
 * be returned if more than one attempt is still available otherwise the status will be 
 * PPSDK_EMV_STATUS_INVALID_PIN_LAST_ATTEMPT. If a severe error occurs so that the transaction
 * should be terminated immediately, the status PPSDK_EMV_STATUS_ABORT_TRANSACTION will be set.
 * If any kind of internal error occurs during the processing, the status 
 * PPSDK_EMV_STATUS_SYSTEM_ERROR will be returned. If the verification cannot be completed due
 * to a missing CA public key, the status returned will be PPSDK_EMV_STATUS_INVALID_KEY. 
 * Finally if the PIN is entered and verified correctly the status got will be 
 * PPSDK_EMV_STATUS_SUCCESS.
 * 
 * The PPSDK_EMV_STATUS_INVALID_KEY status code will let the application to detect and
 * invalid configuration concerning the CA RSA public keys.
 * 
 * The PIN entry process will have to be accomplished by the application calling to the proper
 * commands provided for that aim.
 *
 * @param handler a pointer to the instance handler.  
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emv_verifyPinOFF(void *handler, unsigned short *sw);

// AC type definitions
#define PPSDK_EMV_CERTIFICATE_AAC      = 0
#define PPSDK_EMV_CERTIFICATE_TC       = 1
#define PPSDK_EMV_CERTIFICATE_ARQC     = 2
#define PPSDK_EMV_CERTIFICATE_AC_TYPES = 3

#define PPSDK_EMV_RISK_CDOL_1 1 // First generation of AC
#define PPSDK_EMV_RISK_CDOL_2 2 // Second generation of AC

/**
 * Using this command the application will be able to generate an application cryptogram, the
 * first or the second one, as required by the transaction.
 * 
 * If any of the incoming parameters value is incorrect the status 
 * PPSDK_EMV_STATUS_DATA_FORMAT_ERROR will be returned. The status PPSDK_EMV_STATUS_SYSTEM_ERROR
 * will be get by the application if any internal error occurs during the processing. If during
 * the cryptogram generation an error occurs that requires the transaction termination, the 
 * status PPSDK_EMV_STATUS_ABORT_TRANSACTION will be informed. If other kind of error occurs 
 * during the generation the status PPSDK_EMV_STATUS_ERROR_AC_PROCESS will be got. If the 
 * combined authentication is enabled, PPSDK_EMV_STATUS_CDA_FAILED will be returned to indicate
 * that it failed. Finally if the certificate can be obtained with no error the status will be
 * PPSDK_EMV_STATUS_SUCCESS.
 * 
 * If the CDA is the card data authentication mode the CDA will be always requested on the first
 * cryptogram generation if the cryptogram type to be requested is a TC. It will be always
 * disabled for AAC and for an ARQC depends on the CDA mode active.
 * 
 * If the CDA is the card data authentication mode the CDA will be disabled on the second
 * cryptogram generation if the cryptogram type to be requested is an AAC, otherwise if the
 * cryptogram type is a TC it will depend on the CDA mode active.
 *
 * @param handler a pointer to the instance handler.
 * @param type AC type. The value must be one of PPSDK_EMV_CERTIFICATE_* constant type values.
 * @param risk risk type. The value must be one of PPSDK_EMV_RISK_* constant type values.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvGenerateCertificate(void *handler, unsigned char type, unsigned char risk, unsigned short *sw);

/**
 * The command checks the action codes (provided by the application and read from the card), the
 * TVR and will determine how the transaction is resolved.
 *
 * PPSDK_EMV_STATUS_SYSTEM_ERROR will be got by the application if any internal error occurs 
 * during the processing. First the denial action codes are checked, if any of the bits in 
 * the TVR match the status PPSDK_EMV_STATUS_TRANSACTION_DENIED will be returned, otherwise
 * if the terminal is both offline & online, the online action codes will be checked in the 
 * same way and if any of the bits match with the TVR data the status 
 * PPSDK_EMV_STATUS_TRANSACTION_ONLINE will be set, if theres no match at all the status 
 * will be PPSDK_EMV_STATUS_TRANSACTION_APPROVED instead. If the terminal is offline only 
 * the default action code is checked, if any of the bits in the TVR match the status 
 * PPSDK_EMV_STATUS_TRANSACTION_DENIED will be returned, otherwise the status got will be 
 * PPSDK_EMV_STATUS_TRANSACTION_APPROVED}. If the terminal is online only the status 
 * PPSDK_EMV_STATUS_TRANSACTION_ONLINE} will be returned.
 * 
 * According to the latest EMV recommendations concerning the CDA processing (Specification
 * update bulletin No. 44) if the CDA is the card data authentication mechanism to be performed,
 * the previous key recovery process will be accomplished prior to the transaction decision so
 * that CDA errors could be detected in advance and reflected on the TVR.
 * 
 * The online/offline capability of the terminal is determined by the value of the tag
 * TAG_9F35_TERMINAL_TYPE.
 *
 * @param handler a pointer to the instance handler.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvMakeTransactionDesision(void *handler, unsigned short *sw);

/**
 * The command checks the default action code (provided by the application and read from the
 * card), the TVR and will determine how the transaction is resolved by default.
 *
 * PPSDK_EMV_STATUS_SYSTEM_ERROR will be returned if any internal error occurs during the 
 * processing. If any of the bits in the TVR match with the default action codes the status
 * PPSDK_EMV_STATUS_TRANSACTION_DENIED will be returned, otherwise the status will be
 * PPSDK_EMV_STATUS_TRANSACTION_APPROVED instead.
 *
 * @param handler a pointer to the instance handler.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvMakeDefaultDecision(void *handler, unsigned short *sw);

/**
 * The command is used to validate the cryptogram got from the issuer.
 * 
 * If the issuer cryptogram was not set previously, the status PPSDK_EMV_STATUS_NO_DATA_FOUND
 * will be returned. If during the processing any internal error occurs, the status 
 * PPSDK_EMV_STATUS_SYSTEM_ERROR will be set. If everything is OK and the cryptogram is 
 * verified, the result will be PPSDK_EMV_STATUS_SUCCESS.
 * 
 * The data item that the application has to provide to the kernel so that this command could be
 * executed is: TAG_91_ISSUER_AUTH_DAT.
 *
 * @param handler a pointer to the instance handler.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvAuthenticateIssuer(void *handler, unsigned short *sw);

// EMV script type definitions.
#define PPSDK_EMV_SCRIPT_71  71
#define PPSDK_EMV_SCRIPT_72  72

/**
 * The script processing retrieved in the online authorization is handled by this command.
 *
 * First the presence of the script in the data repository is checked, if its not present the
 * status PPSDK_EMV_STATUS_NO_SCRIPT_LOADED is returned. If during the processing any internal
 * error occurs the status PPSDK_EMV_STATUS_SYSTEM_ERROR will be set. Once  the script has been
 * conveniently processed and issued to the card the status PPSDK_EMV_STATUS_SUCCESS will be set.
 * 
 * The script data should be provided to the kernel through the data item TAG_DF24_ISSUER_SCRIPTS,
 * and after the processing is over the results can be recovered by accessing the data item 
 * TAG_DF25_ISSUER_SCRIPTS_RESULT}. The maximum length of the scripts supported is 256 bytes.
 *
 * @param handler a pointer to the instance handler.
 * @param script script type. The value must be one of PPSDK_EMV_SCRIPT_* constant type values.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvScriptProcessing(void *handler, unsigned char script, unsigned short *sw);

/**
 * The command allows modifying the TVR directly, setting or clear the desired bits.
 * 
 * Initially the incoming parameters are validated to ensure that are pointing to a valid
 * location within the TVR structure. If thats not the case, the status
 * PPSDK_EMV_STATUS_DATA_FORMAT_ERROR will be returned. If during the processing any
 * internal error occurs, the status PPSDK_EMV_STATUS_SYSTEM_ERROR will be set.
 * PPSDK_EMV_STATUS_SUCCESS} will be returned if everything is correct and the TVR could be
 * updated.
 * 
 * The aim of this command is to let the application to achieve any additional procedure that
 * could need as a particular requirement. Consult section List of TVR and TSI bits below for
 * the complete list of the bits.
 *
 * @param handler a pointer to the instance handler.
 * @param byteIdx the byte index to update, which can be in the range (1 ~ 5).
 * @param bitIdx the bit index to update, which can be in the range (1 ~ 8).
 * @param value the bit value, which can be 0 or 1. 
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvUpdateTVR(void *handler, unsigned char byteIdx, unsigned char bitIdx, unsigned char value, unsigned short *sw);

/**
 * The command allows modifying the TSI directly, setting or clear the desired bits.
 * 
 * Initially the incoming parameters are validated to ensure that are pointing to a valid
 * location within the TSI structure, if thats not the case the status
 * PPSDK_EMV_STATUS_DATA_FORMAT_ERROR will be returned. If during the processing any
 * internal error occurs the status PPSDK_EMV_STATUS_SYSTEM_ERROR will be set.
 * PPSDK_EMV_STATUS_SUCCESS will be returned if everything is correct and the TVR
 * could be updated.
 * 
 * The aim of this command is to let the application to achieve any additional procedure that
 * could need as a particular requirement. Consult section List of TVR and TSI bits for a list
 * of the bits.
 *
 * @param handler a pointer to the instance handler.
 * @param byteIdx the byte index to update, which can be in the range (1 ~ 5).
 * @param bitIdx the bit index to update, which can be in the range (1 ~ 8).
 * @param value the bit value, which can be 0 or 1. 
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvUpdateTSI(void *handler, unsigned char byteIdx, unsigned char bitIdx, unsigned char value, unsigned short *sw);

/**
 * The command is intended to verify an individual bit within the TVR.
 * <p>
 * Initially the incoming parameters are validated to ensure that are pointing to a valid
 * location within the TVR structure, if thats not the case the status
 * PPSDK_EMV_STATUS_DATA_FORMAT_ERROR will be returned. If during the processing any
 * internal error occurs the status PPSDK_EMV_STATUS_SYSTEM_ERROR} will be set.
 * PPSDK_EMV_STATUS_SUCCESS will be returned if the given bit is set otherwise it will
 * be PPSDK_EMV_STATUS_FAILURE.
 * 
 * The aim of this command is to let the application to achieve any additional procedure that
 * could need as a particular requirement. Consult section List of TVR and TSI bits for a list
 * of the bits.
 *
 * @param handler a pointer to the instance handler.
 * @param byteIdx the byte index to update, which can be in the range (1 ~ 5).
 * @param bitIdx the bit index to update, which can be in the range (1 ~ 8).
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvCheckTVR(void *handler, unsigned char byteIdx, unsigned char bitIdx, unsigned short *sw);

/**
 * The command is intended to verify an individual bit within the TSI.
 * 
 * Initially the incoming parameters are validated to ensure that are pointing to a valid
 * location within the TSI structure, if thats not the case the status
 * PPSDK_EMV_STATUS_DATA_FORMAT_ERROR will be returned. If during the processing any
 * internal error occurs the status PPSDK_EMV_STATUS_SYSTEM_ERROR will be set.
 * PPSDK_EMV_STATUS_SUCCESS will be returned if the given bit is set otherwise it will
 * be PPSDK_EMV_STATUS_FAILURE.
 * 
 * The aim of this command is to let the application to achieve any additional procedure that
 * could need as a particular requirement. Consult section List of TVR and TSI bits for a list
 * of the bits.
 *
 * @param handler a pointer to the instance handler.
 * @param byteIdx the byte index to update, which can be in the range (1 ~ 5).
 * @param bitIdx the bit index to update, which can be in the range (1 ~ 8).
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvCheckTSI(void *handler, unsigned char byteIdx, unsigned char bitIdx, unsigned short *sw);

/**
 *  The command load public key.
 *
 * @param handler a pointer to the instance handler.   
 * @param rid a registered application provider identifier (RID).
 * @param idx key index. 
 * @param mod a buffer that contains key module.
 * @param modLen key module length.
 * @param exp a buffer that contains key exponent.
 * @param expLen key exponent length. 
 * @param bitIdx the bit index to update, which can be in the range (1 ~ 8).
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvLoadPublicKey(void *handler, const unsigned char rid[5], unsigned char idx, 
								 const unsigned char mod[], unsigned int modLen, 
								 const unsigned char exp[], unsigned int expLen, 
								 const unsigned char hash[20], unsigned short *sw);

/**
 * The command is intended to delete a given CA public key.
 * 
 * If the input is empty the status returned will be PPSDK_EMV_STATUS_NO_DATA_FOUND,
 * if the key cannot be found the PPSDK_EMV_STATUS_INVALID_KEY} status will be got. If
 * during the processing any internal error occurs the returned status will be
 * PPSDK_EMV_STATUS_SYSTEM_ERROR. Finally if the key can be deleted the status will be
 * PPSDK_EMV_STATUS_SUCCESS.
 *
 * @param handler a pointer to the instance handler.   
 * @param rid a registered application provider identifier (RID).
 * @param idx key index.  
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvRemovePublicKey(void *handler, const unsigned char rid[5], unsigned char idx, unsigned short *sw);

/**
 * The command sets a data item with data in binary format (raw data).
 * 
 * Initially the input data is validated, if the buffer is empty the status 
 * PPSDK_EMV_STATUS_NO_DATA_FOUND will be returned, in case of not locating the tag
 * PPSDK_EMV_STATUS_NO_SCRIPT_LOADED will be set, if the length of the incoming data
 * is not in the range accepted by the data item the status PPSDK_EMV_STATUS_INVALID_LENGTH
 * will be returned. The data item attributes are checked to determine whether the item can 
 * be written or not, if its not the case the status returned will be 
 * PPSDK_EMV_STATUS_INVALID_TAG. If during the processing any internal error occurs the 
 * returned status will be PPSDK_EMV_STATUS_SYSTEM_ERROR. Once the data has been saved 
 * properly the status PPSDK_EMV_STATUS_SUCCESS will be set.
 * 
 * Using this method theres no applicable conversion, so the data provided should be in the
 * format that corresponds to the data item to be set. So, in fact, its like setting a given
 * data item with raw data. Consult section List of EMV tags for a list of the data items.
 *
 * @param handler a pointer to the instance handler.   
 * @param tag the tag value.
 * @param data a buffer that contains tag data.
 * @param dataLen data length.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvSetDataAsBinary(void *handler, unsigned short tag, 
								   const unsigned char data[], unsigned int dataLen, unsigned short *sw);

/**
 * The command sets a data item with data in string format.
 * <p>
 * Initially the input data is validated, if the buffer is empty the status
 * PPSDK_EMV_STATUS_NO_DATA_FOUND will be returned, in case of not locating the tag
 * PPSDK_EMV_STATUS_NO_SCRIPT_LOADED} will be set, if the length of the incoming data
 * is not in the range accepted by the data item the status PPSDK_EMV_STATUS_INVALID_LENGTH
 * will be returned. The data item attributes are checked to determine whether the item can 
 * be written or not, if its not the case the status returned will be 
 * PPSDK_EMV_STATUS_INVALID_TAG. If during the processing any internal error occurs the 
 * returned status will be PPSDK_EMV_STATUS_SYSTEM_ERROR. Once the data has been saved properly
 * the status PPSDK_EMV_STATUS_SUCCESS} will be set.
 * 
 * Using this method theres no applicable conversion, so the data provided should be in the
 * format that corresponds to the data item to be set. So, in fact, its like setting a given
 * data item with raw data. Consult section List of EMV tags for a list of the data items.
 *
 * @param handler a pointer to the instance handler.   
 * @param tag the tag value.
 * @param data a buffer that contains NULL terminated tag data. 
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvSetDataAsString(void *handler, unsigned short tag, const char data[], unsigned short *sw);

/**
 * The command gets a data item in binary format (raw data).
 * 
 * If the length of the data item is greater than the length of the buffer requested the status
 * PPSDK_EMV_STATUS_EMV_INVALID_LENGTH will be set, in the case of not finding the requested item
 * the status PPSDK_EMV_STATUS_EMV_NO_SCRIPT_LOADED} will be returned. After checking the item 
 * attributes, if the item cannot be read the returned status will be 
 * PPSDK_EMV_STATUS_EMV_INVALID_TAG. If during the processing any internal error occurs the 
 * returned status will be PPSDK_EMV_STATUS_EMV_SYSTEM_ERROR. Finally if everything is OK and the 
 * data can be extracted the status will be PPSDK_EMV_STATUS_EMV_SUCCESS.
 * 
 * Using this method theres no applicable conversion, so the data retrieved is in the format
 * that corresponds to the data item. Consult section List of EMV tags for a list of the data
 * items.
 *
 * @param handler a pointer to the instance handler.   
 * @param tag the tag value.
 * @param out a buffer that receives tag data.
 * @param outMaxSize the maximum length of received data.
 * @param nbr a pointer to value that receives number of bytes stored in output buffer.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvGetDataAsBinary(void *handler, unsigned short tag, 
								   unsigned char out[], unsigned int outMaxLen, unsigned int *nbr, unsigned short *sw);

/**
 * The command gets a data item in string format.
 * 
 * If the length of the data item is greater than the length of the buffer requested the status
 * PPSDK_EMV_STATUS_EMV_INVALID_LENGTH will be set, in the case of not finding the requested item
 * the status PPSDK_EMV_STATUS_EMV_NO_SCRIPT_LOADED will be returned. After checking the item 
 * attributes, if the item cannot be read the returned status will be 
 * PPSDK_EMV_STATUS_EMV_INVALID_TAG. If during the processing any internal error occurs
 * the returned status will be PPSDK_EMV_STATUS_EMV_SYSTEM_ERROR. Finally if everything is
 * OK and the data can be extracted the status will be PPSDK_EMV_STATUS_EMV_SUCCESS.
 * 
 * Using this method theres no applicable conversion, so the data retrieved is in the format
 * that corresponds to the data item. Consult section List of EMV tags for a list of the data
 * items.
 *
 * @param handler a pointer to the instance handler.   
 * @param tag the tag value.
 * @param out a buffer that receives tag data, including terminating zero.
 * @param outMaxSize the maximum length of received data. 
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvGetDataAsString(void *handler, unsigned short tag, char out[], unsigned int outMaxLen, unsigned short *sw);

#define PPSDK_EMV_TAG_BINARY  0 // Encode as binary value.
#define PPSDK_EMV_TAG_BCD     1 // Encode as BCD value.
#define PPSDK_EMV_TAG_ASCIIZ  2 // Encode as string value.

/**
 * The command allows the application direct access to the data of a given item.
 * 
 * In the case of not finding the requested item the status PPSDK_EMV_STATUS_NO_SCRIPT_LOADED
 * will be returned. If during the processing any internal error occurs, the returned status 
 * will be PPSDK_EMV_STATUS_SYSTEM_ERROR. Finally, if everything is OK and the attributes can
 * be extracted, the status will be PPSDK_EMV_STATUS_SUCCESS.
 * 
 * The aim of this command is to let the application a direct access to the already assigned
 * buffers of the data items. This could be useful to save and to optimize memory usage. It can
 * be also used to determine the presence of an item or to know its current length.
 *
 * @param handler a pointer to the instance handler.   
 * @param tag the tag value.
 * @param type a pointer to variable that receives tag type. The value can be one of 
 *             PPSDK_EMV_TAG_* constant type values.
 * @param maxLen a pointer to variable that receives the maximum length of tag. 
 * @param curLen a pointer to variable that receives the current length of tag. 
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvGetDataDetails(void *handler, unsigned short tag, unsigned char *type, 
								  unsigned int *maxLen, unsigned int *curLen, unsigned short *sw);

// Bypass method mode definitions.
#define PPSDK_EMV_BYPASS_CURRENT_METHOD_MODE  0
#define PPSDK_EMV_BYPASS_ALL_METHODS_MODE     1

/**
 * The command allows the application direct access to the data of a given item.
 * 
 * With this command is possible to setup the behavior of the KERNEL regarding the PIN based
 * method bypass, so that only the current method will be bypassed or any other found later in
 * the CVM list will be considered so as well.
 * 
 * If any kind of internal error occurs during the processing or the kernel was not initialized
 * before the status PPSDK_EMV_STATUS_EMV_SYSTEM_ERROR will be returned. On the other hand
 * if the value can be set correctly the status got will be PPSDK_EMV_STATUS_EMV_SUCCESS.
 *  
 * If this command is not used along the transaction the default value applied by the kernel
 * will be PPSDK_EMV_BYPASS_CURRENT_METHOD_MODE. If the expected behavior is other than the
 * default one the call to this command will have to be done prior to the card holder
 * authentication procedure and after application selection.
 *
 * @param handler a pointer to the instance handler.   
 * @param mode bypass mode. The value must be one of PPSDK_EMV_BYPASS_* constant type values. 
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvSetBypassMode(void *handler, unsigned char mode, unsigned short *sw);

/**
 * Set multiple of tags. 
 *  
 * @param handler a pointer to the instance handler.   
 * @param tlv a buffer that contains list of BER TLV structures.
 * @param tlvLen tlv data length.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvSetTags(void *handler, const unsigned char tlv[], unsigned int tlvLen, unsigned short *sw);

/**
 * Read multiple of tags.
 *  
 * @param handler a pointer to the instance handler.   
 * @param tag a buffer that contains list of tags.
 * @param tagLen tag data length.
 * @param tlv a buffer that receives list of BER TLV structures.
 * @param tlvLen tlv data length.
 * @param nbr a pointer to variable that receives number of bytes stored in tlv buffer.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvGetTags(void *handler, const unsigned char tag[], unsigned int tagLen, 
						   unsigned char tlv[], unsigned int tlvMaxLen, unsigned int *nbr,
						   unsigned short *sw);

/**
 * Read multiple of tags encrypted with 3DES CBC.
 *  
 * @param handler a pointer to the instance handler.   
 * @param key a key number (1 ~ 49). A value of 0xFF use last generated DUKPT key.
 * @param id the 4 bytes unique identifier.
 * @param tag a buffer that contains list of tags.
 * @param tagLen tag data length.
 * @param data a buffer that receives encrypted BER TLV structures.
 * @param dataMaxLen encrypted tlv data length.
 * @param nbr a pointer to variable that receives number of bytes stored in data buffer.
 * @param sw a pointer to variable that receives emv status of command execution. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emvGetTagsEncrypted(void *handler, unsigned char key, unsigned int id, const unsigned char tag[], unsigned int tagLen, 
						   unsigned char data[], unsigned int dataMaxLen, unsigned int *nbr,
						   unsigned short *sw);

////////////////////////////////////////////////////////////////////////////////
// BARCODE COMMANDS
////////////////////////////////////////////////////////////////////////////////

/**
 * Power on barcode engine.
 *    
 * @param handler a pointer to the instance handler. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_bcPowerOn(void *handler);

/**
 * Power off barcode engine.
 *    
 * @param handler a pointer to the instance handler. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_bcPowerOff(void *handler);

/**
 * Checks if barcode data is available.
 *   
 * @param handler a pointer to the instance handler. 
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_bcPull(void *handler);

/**
 * Get data of scanned barcode.
 *   
 * @param handler a pointer to the instance handler. 
 * @param out a buffer that receives output data.
 * @param outMaxLen the maximum length of received data.
 * @param nbr a pointer to variable that receives number of bytes stored in output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_bcGetData(void *handler, unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

// The barcode scan mode
#define PPSDK_SCAN_MODE_SINGLE     1  // Single scan mode.
#define PPSDK_SCAN_MODE_MULTIPLE   1  // Multiple scan mode.

/**
 * Start reading.
 *   
 * @param handler a pointer to the instance handler. 
 * @param mode the barcode mode. The value must be one of PPSDK_SCAN_MODE_* constant type values.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_bcStartRead(void *handler, unsigned char mode);

/**
 * Send initial barcode commands.
 *   
 * @param handler a pointer to the instance handler. 
 * @param data a buffer that contains command data.
 * @param data data length.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_bcSetInitCommand(void *handler, const unsigned char data[], unsigned int dataLen);

/**
 * Set to play sound when barcode is read.
 *  
 * To disable sound set duration to 0.
 *
 * @param handler a pointer to the instance handler.  
 * @param freq the tone frequency in Hz.
 * @param duration the tone duration in miliseconds.
 * @param volume the tone volume form 0 to 100%.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_bcSetSound(void *handler, unsigned int freq, unsigned int duration, unsigned char volume);


////////////////////////////////////////////////////////////////////////////////
// EMV2
////////////////////////////////////////////////////////////////////////////////
/**
 * AES 256 encryption algorithm
 */
#define PPSDK_EMV2_ALG_AES256		0
/**
 * Encrypted Head ECC encryption algorithm
 */
#define PPSDK_EMV2_ALG_EH_ECC     1
/**
 * Encrypted Head AES256 encryption algorithm. Encryption type is CBC. After decryption, the
 * result data will be as follows: - Random data (4 bytes) - Device identification text (16
 * ASCII characters, unused bytes are 0) - Processed track data in the format: 0xF1 (track1
 * data), 0xF2 (track2 data) 0xF3 (track3 data). It is possible some of the tracks will be
 * empty, then the identifier will not be present too, for example 0xF1 (track1 data) 0xF3
 * (track3 data) - End of track data (byte 0x00) - CRC16CCIT (2 bytes) - the CRC is performed
 * from the start of the encrypted block (the Random Data block) to the end of the track data
 * (including the 0x00 byte). The data block is rounded to 16 bytes
 */
#define PPSDK_EMV2_ALG_EH_AES256  2
/**
 * Encrypted Head IDTECH encryption algorithm, please refer to IDTECH documentation for detailed
 * format and examples. Data, that is received via magneticCardEncryptedData has the following
 * format: - (1 byte) card encoding type, can ignore - (1 byte) bits marking which track is
 * present - (1 byte) track 1 UNENCRYPTED length - (1 byte) track 2 UNENCRYPTED length - (1
 * byte) track 3 UNENCRYPTED length - (track 1 UNENCRYPTED length bytes) track 1 masked data -
 * (track 2 UNENCRYPTED length bytes) track 2 masked data - (track 3 UNENCRYPTED length bytes)
 * track 3 masked data - (variable bytes) Track 1 + Track 2 encrypted data, the length of this
 * block is calculated by subtracting from the end - (20 bytes) track 1 sha1 - (20 bytes) track
 * 2 sha1 - (10 bytes) KSN
 * 
 * Encrypted block contents after decryption (3DES): - (track 1 UNENCRYPTED length bytes) track
 * 1 data - (track 2 UNENCRYPTED length bytes) track 2 data
 */
#define PPSDK_EMV2_ALG_EH_IDTECH  3
/**
 * Encrypted Head MAGTEK encryption algorithm
 */
#define PPSDK_EMV2_ALG_EH_MAGTEK  4
/**
 * Encrypted Head 3DES encryption algorithm
 */
#define PPSDK_EMV2_ALG_EH_3DES    5

/**
 * Authentication key
 */
#define PPSDK_EMV2_KEY_AUTHENTICATION           0x00

/**
 * Encrypted head key loading(encryption) key (KEK), used to load other keys with
 */
#define PPSDK_EMV2_KEY_EH_AES256_LOADING        0x02

/**
 * Encrypted head AES256 Data encryption key 1
 */
#define PPSDK_EMV2_KEY_EH_AES256_ENCRYPTION1    0x01
/**
 * Encrypted head AES256 Data encryption key 2 (present on EMSR firmware version 2.30 and above)
 */
#define PPSDK_EMV2_KEY_EH_AES256_ENCRYPTION2    0x03
/**
 * Encrypted head AES256 Data encryption key 3 (present on EMSR firmware version 2.30 and above)
 */
#define PPSDK_EMV2_KEY_EH_AES256_ENCRYPTION3    0x04

/**
 * Encrypted head AES128 Data encryption key 1 (present on EMSR firmware version 2.30 and above)
 */
#define PPSDK_EMV2_KEY_EH_AES128_ENCRYPTION1    0x09
/**
 * Encrypted head AES128 Data encryption key 2 (present on EMSR firmware version 2.30 and above)
 */
#define PPSDK_EMV2_KEY_EH_AES128_ENCRYPTION2    0x0B
/**
 * Encrypted head AES128 Data encryption key 3 (present on EMSR firmware version 2.30 and above)
 */
#define PPSDK_EMV2_KEY_EH_AES128_ENCRYPTION3    0x0C

/**
 * Encrypted head TMK key
 */
#define PPSDK_EMV2_KEY_EH_TMK_AES         0x10

/**
 * Encrypted head 3DES DUKPT data encryption key 1
 */
#define PPSDK_EMV2_KEY_EH_DUKPT_MASTER1   0x20
/**
 * Encrypted head 3DES DUKPT data encryption key 2 (present on EMSR firmware version 2.30 and
 * above)
 */
#define PPSDK_EMV2_KEY_EH_DUKPT_MASTER2   0x21
/**
 * Encrypted head 3DES DUKPT data encryption key 3 (present on EMSR firmware version 2.30 and
 * above)
 */
#define PPSDK_EMV2_KEY_EH_DUKPT_MASTER3   0x22
   

/**
 * Load EMV configuration.
 * 
 * @param handler a pointer to the instance handler.  
 * @param type configuration type, one of PPSDK_EMV2_CONFIG_* constant values.
 * @param data configuration data.
 * <pre>
 * - Template 0xE0 - Terminal configuration
 * - Template 0xE1 - Application configuration (one for each application)
 * - Template 0xE4 - Information
 *     - Tag 0xC1 - Configuration version - 4 bytes   (big-endian)
 * - Template 0xE5 - TAG Lists
 *      - Tag 0xC1 - Tags that will be returned in 'ONLINE' stage
 *      - Tag 0xC2 - Tags that will be returned in 'COMPLETE' stage
 * </pre>
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emv2LoadConfiguration(void *handler, int type, const unsigned char data[], unsigned int dataLen);

/**
 * Loads EMV kernel contactless configuration data.
 * <p>
 * Configuration consists of custom tags setting various terminal capabilities and specific
 * application parameters.
 * 
 * @param handler a pointer to the instance handler.  
 * @param data TLV list of configuration tags. If the configuration is loaded for the first
 *            time, or there is no configuration encryption key tag set, then it is possible to
 *            load it in plain.
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emv2LoadContactlessConfiguration(void *handler, const unsigned char data[], unsigned int dataLen);

/**
 * Loads EMV kernel contact/chip configuration data.
 * <p>
 * Configuration consists of custom tags setting various terminal capabilities and specific
 * application parameters.
 * 
 * @param handler a pointer to the instance handler.  
 * @param handler a pointer to the instance handler.  
 * @param data TLV list of configuration tags. If the configuration is loaded for the first
 *            time, or there is no configuration encryption key tag set, then it is possible to
 *            load it in plain.
 * 
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emv2LoadContactConfiguration(void *handler, const unsigned char data[], unsigned int dataLen);

/**
 * Loads EMV kernel contactless Certification Authority Public Keys.
 * 
 * @param handler a pointer to the instance handler.  
 * @param data TLV list of configuration tags containing public keys parameters.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emv2LoadContactlessCAPK(void *handler, const unsigned char data[], unsigned int dataLen);

/**
 * Loads EMV kernel contact/chip Certification Authority Public Keys.
 * 
 * @param handler a pointer to the instance handler.  
 * @param data TLV list of configuration tags containing public keys parameters.
 *
 * @return On success returns 0; otherwise, returns a positive error number.
 */
PPSDK_ERR ppsdk_emv2LoadContactCAPK(void *handler, const unsigned char data[], unsigned int dataLen);

/**
 * EMV2 configuration information. 
 */ 
typedef struct {
	int configurationVersion;
    int contactlessConfigurationVersion;
    int contactConfigurationVersion;
    int contactlessCAPKVersion;
    int contactCAPKVersion;
    int emvKernelVersion;
} PPSDK_EMV2Info;

/**
 * Returns information about loaded configuration;
 * 
 * @param handler a pointer to the instance handler.  
 * @param info A pointer to structure that receives EMV2 information.
 *
 * @return On success returns 0; otherwise, returns a positive error number. 
 */
PPSDK_ERR ppsdk_emv2GetInfo(void *handler, PPSDK_EMV2Info* info);

/**
 * EMV chip card configuration.
 */
#define PPSDK_EMV2_CONFIG_CHIP         0
/**
 * EMV contactless card configuration.
 */
#define PPSDK_EMV2_CONFIG_CONTACTLESS  1
/**
 * EMV chip card CA keys.
 */
#define PPSDK_EMV2_CONFIG_CHIP_CA_KEYS  2
/**
 * EMV contactless card CA keys.
 */
#define PPSDK_EMV2_CONFIG_CL_CA_KEYS    3

/**
 * Payment Interface: Contact SmartCard interface
 */
#define PPSDK_EMV2_INTERFACE_CONTACT    1
/**
 * Payment Interface: Contactless interface
 */
#define PPSDK_EMV2_INTERFACE_CONTACTLESS  2
/**
 * Payment Interface: Contact Magnetic Card interface
 */
#define PPSDK_EMV2_INTERFACE_MAGNETIC     4
/**
 * Payment Interface: Manual card entry
 */
#define PPSDK_EMV2_INTERFACE_MANUAL  8

/**
 * CVM Result: Online PIN.
 */
#define PPSDK_EMV2_CVM_ONLINE_PIN  0x01
/**
 * CVM Result: Configuration code verified.
 */
#define PPSDK_EMV2_CVM_CONFIG_CODE_VERIFIED 0x02
/**
 * CVM Result: Signature needed.
 */
#define PPSDK_EMV2_CVM_OBTAIN_SIGNATURE     0x03
/**
 * CVM Result: No CVM was performed.
 */
#define PPSDK_EMV2_CVM_NO_CVM               0x04
/**
 * CVM Result: Offline PIN was entered.
 */
#define PPSDK_EMV2_CVM_OFFLINE_PIN          0x08

/**
 * Transaction result: Transaction is aborted.
 */
#define PPSDK_EMV2_RESULT_ABORTED           0xFFFF
/**
 * Transaction result: Transaction is approved.
 */
#define PPSDK_EMV2_RESULT_APPROVED          0x0800
/**
 * Transaction result: Transaction is declined.
 */
#define PPSDK_EMV2_RESULT_DECLINED          0x0400
/**
 * Transaction result: Try another interface.
 */
#define PPSDK_EMV2_RESULT_TRY_ANOTHER_INTERFACE 0x0100
/**
 * Transaction result: Wrong card application or configuration.
 */
#define PPSDK_EMV2_RESULT_END_APPLICATION       0x0000
/**
 * Transaction result: Try again
 */
#define PPSDK_EMV2_RESULT_TRY_AGAIN             0x1000

/**
 * Transaction card reading failed.
 */
#define PPSDK_EMV2_FAILURE_REASON_FAILED  0
/**
 * Transaction timed out.
 */
#define PPSDK_EMV2_FAILURE_REASON_TIMEOUT  1
/**
 * User canceled the transaction.
 */
#define PPSDK_EMV2_FAILURE_REASON_CANCELED  2
/**
 * Customer's card is blocked
 */
#define PPSDK_EMV2_FAILURE_REASON_CARD_BLOCKED  3
/**
 * Double tap transaction
 */
#define PPSDK_EMV2_FAILURE_REASON_DOUBLE_TAP_TRANSACTION 4
/**
 * Card read failure
 */
#define PPSDK_EMV2_FAILURE_REASON_CARD_READ_FAILED    99


/**
 * DUKPT encryption algorithm returning the data in:
 * <ul>
 * <li>Data length inside the encrypted block (2 bytes, MSB), i.e. if only SW1 and SW2 are
 * inside, the length will be 2 while the encrypted block - 8 </li>
 * <li>Encrypted apdu response (variable, 00 for padding)</li>
 * <li>SW1 and SW2 in plain (2 bytes)</li>
 * <li>KSN (10 bytes)</li>
 * </ul>
 */
#define PPSDK_ALG_APDU_DUKPT     0

/**
 * Starts EMV transaction. The function waits for payment card to be available, then processes
 * it and notifies of completion. You can cancel the transaction at any time.
 * 
 * @param handler a pointer to the instance handler.  
 * @param interfaces transaction interface like contactless/contact or magnetic. One of the
 *            PPSDK_EMV2_INTERFACE_* constants, only works on universal EMV kernel.
 * @param flags controls the EMV transaction, use 0 for now.
 * @param data Pointer to the buffer that contains optional TLV structure with additional 
 * parameters to be sent to the kernel.
 * @param dataLen data buffer length.
 * @param timeout transaction timeout in seconds, only works on universal EMV kernel.
 *
 * @return On success returns 0; otherwise, returns a positive error number. 
 */
PPSDK_ERR ppsdk_emv2StartTransaction(void *handler, int interfaces, int flags, const unsigned char data[], 
									 unsigned int dataLen, int timeout);

/**
 * Responds to the EMV kernel after an online request was sent to the card and the communication
 * with the financial institution is complete.
 * 
 * @param handler a pointer to the instance handler.  
 * @param tags Pointer to buffer that contains the input tag data.
 * 
 * <pre>
 * - Tag 0xC2 - Online success     - 1 byte
 *                                 - True(1) or False(0)
 * - Template 0xE6  - Host response tags.
 * </pre>
 * @param tagsLen tags buffer length.
 *
 * @return On success returns 0; otherwise, returns a positive error number. 
 */
PPSDK_ERR ppsdk_emv2SetOnlineResult(void *handler, const unsigned char tags[], unsigned int tagsLen);

/**
 * Cancels an active EMV operation and clears all data. Using this function is not required if
 * the EMV transaction completed.
 * 
 * @param handler a pointer to the instance handler.  
 *
 * @return On success returns 0; otherwise, returns a positive error number. 
 */
PPSDK_ERR ppsdk_emv2CancelTransaction(void *handler);

/**
 * This command initializes the EMV kernel, call it before calling any other EMV function.
 * 
 * @param handler a pointer to the instance handler.  
 *
 * @return On success returns 0; otherwise, returns a positive error number. 
 */
PPSDK_ERR ppsdk_emv2Initialize(void *handler);

/**
 * This command deinitialise the EMV kernel and frees the allocated resources, call it after
 * you are done with the EMV transaction.
 * 
 * @param handler a pointer to the instance handler.  
 *
 * @return On success returns 0; otherwise, returns a positive error number. 
 */
PPSDK_ERR ppsdk_emv2Deinitialise(void *handler);

/**
 * After transaction is finished, you can get the card data in magnetic-stripe format by using
 * one of the available Encrypted Head formats.
 * 
 * @param format one of the PPSDK_TAGS_FORMAT_* constants.
 * @param keyID key ID, one of PPSDK_EMV2_KEY_* constants.
 * @param out pointer to buffer that receives encrypted block as formatted and encrypted by Emulated EMSR module.
 * @param outMaxLen output buffer length.
 * @param nbr pointer to variable that receives number of bytes returns into output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number. 
 */
PPSDK_ERR ppsdk_emv2GetTrackData(void *handler, int format, int keyID, unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

/**
 * After transaction is finished, you can get the card data in magnetic-stripe format by using
 * one of the available Encrypted Head formats.
 *
 * @param handler a pointer to the instance handler.  
 * @param format one of the PPSDK_TAGS_FORMAT_* constants.
 * @param keyID key ID, one of PPSDK_EMV2_KEY_* constants.
 * @param showExpiration returns card expiration date.
 * @param unmaskedDigitsAtStart number of unmasked symbols at the beginning.
 * @param unmaskedDigitsAtEnd number of unmasked symbols at the end.
 * @param out pointer to buffer that receives encrypted block as formatted and encrypted by Emulated EMSR module.
 * @param outMaxLen output buffer length.
 * @param nbr pointer to variable that receives number of bytes returns into output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number. 
 */
PPSDK_ERR ppsdk_emv2GetTrackData2(void *handler, int format, int keyID, int showExpiration, int
        unmaskedDigitsAtStart, int unmaskedDigitsAtEnd, 
		unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

/**
 * After transaction is finished, you can get the card data in magnetic-stripe format by using
 * one of the available Encrypted Head formats.
 *
 * @param handler a pointer to the instance handler.  
 * @param format one of the PPSDK_TAGS_FORMAT_* constants.
 * @param keyID key ID, one of PPSDK_EMV2_KEY_* constants.
 * @param sessionID user defined session identifier.
 * @param out pointer to buffer that receives encrypted block as formatted and encrypted by Emulated EMSR module.
 * @param outMaxLen output buffer length.
 * @param nbr pointer to variable that receives number of bytes returns into output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number. 
 */
PPSDK_ERR ppsdk_emv2GetTrackData3(void *handler, int format, int keyID, int sessionID,
							   unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

/**
 * After transaction is finished, you can get the tags, encrypted and in a specified format.
 * @param tags pointer to buffer that contains a list of tags to get. The format is like TLV list 
 *        without length and value, i.e. every tag takes as many bytes as needed.
 *
 * @param handler a pointer to the instance handler.  
 * @param tagsLen tag buffer length.
 * @param format one of the PPSDK_TAGS_FORMAT_* constants.
 * @param keyType one of the PPSDK_KEY_TYPE_* constants.
 * @param keyIndex key index.
 * @param packetID unique packet identifier.
 * @param out pointer to buffer that receives encrypted block formated and encrypted as specified 
 *            by the format tag.
 * @param outMaxLen output buffer length.
 * @param nbr pointer to variable that receives number of bytes returns into output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number. 
 */
PPSDK_ERR ppsdk_emv2GetTagsEncrypted(void *handler, const unsigned char tags[], unsigned int tagsLen, 
								   int format, int keyType, int keyIndex, int packetID,
								   unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

/**
 * After transaction is finished, you can get the tags in plain. Only non-sensitive tags can be
 * retrieved in plain, no pan/discretionary data will be returned
 * 
 * @param handler a pointer to the instance handler.  
 * @param tags the list of tags to be returned.
 * @return plain TLV data block.
 * @param out pointer to buffer that receives tags in plain.
 * @param outMaxLen output buffer length.
 * @param nbr pointer to variable that receives number of bytes returns into output buffer.
 *
 * @return On success returns 0; otherwise, returns a positive error number. 
 */
PPSDK_ERR ppsdk_emv2GetTagsPlain(void *handler, const unsigned char tags[], unsigned int tagsLen,
								 unsigned char out[], unsigned int outMaxLen, unsigned int *nbr);

#ifdef __cplusplus
}
#endif

#endif // _PINPAD_H

